#!/usr/bin/perl -T
#!/usr/bin/perl -Tw broke when I made changes to list_dtlog that involved
# submitting three commas ",,," in a row into the value of $args :(
#
# NAME
#  mon.cgi
#
#
# DESCRIPTION
#  Web interface for the Mon resource monitoring system. mon.cgi
#  implements a significant subset of the Perl interface to Mon, which
#  allows administrators to quickly view the status of their network
#  and perform many common Mon tasks with a simple web client.
#
# Requires mon 0.38-21 and Mon::Client 0.11 for proper operation.
#
#
# AUTHORS
#  Originally by:
#   Arthur K. Chan <artchan@althem.com>
#  Based on the Mon program by Jim Trocki <trockij@arctic.org>. 
#   http://www.kernel.org/software/mon/
#  Rewritten to support Mon::Client, mod_perl, taint mode,
#  authentication, the strict pragma, and other visual/functional 
#  enhancements by Andrew Ryan <andrewr@nam-shub.com>.
#  Downtime logging contributed by Martha H Greenberg <marthag@mit.edu>
#  Site-specific customization routines contributed by Ed Ravin <eravin@panix.com>
#
# ----------------------------------------------------------------------
# $Id: mon.cgi,v 1.4.2.1 2007/05/03 19:55:38 trockij Exp $
# ----------------------------------------------------------------------
#
#
# INSTRUCTIONS
# Install this cgi script to wherever your cgi-bin directory sits
# on your mon server. If you don't have a web server installed, try
# http://www.apache.org. This script hasn't been tested with any
# web server, although there is no reason it wouldn't work under
# any other web server as a CGI script.
#
# This script now runs cleanly under mod_perl (tested under apache 1.3.9,
# mod_perl 1.21), if you're running that. Global variables have not
# been eliminated but at least we're being careful now.
#
# This script also runs cleanly under taint mode, which is activated
# by using the -T switch for CGI scripts, and by using the directive
# "PerlTaintCheck On" in your httpd.conf file if you are running
# under mod_perl.
#
# Modify the "Configurable Parameters" section below to customize it
# to your site's settings. mon.cgi also supports an optional config
# file which allows you to set all the same parameters. Please
# see the file README.site-customization for more details.
#
# If you want to easily customize the look and feel of mon.cgi, 
# as well as various other configuration options, copy the sample 
# mon.cgi.cf file (in the /config directory of this distribution) 
# into a location where your webserver can read it, and edit the 
# line beginning '$moncgi_config_file = ""' to reflect the path 
# to your config file. You can then change the look and feel of 
# mon.cgi, as well as implement access controls, directly from this 
# file.
#
# If you want to do a lot of the things that this script lets you do,
# and you don't want any authorization to be necessary, then
# you need to open up your auth.cf file to allow anyone to perform
# actions that you would like mon.cgi to perform. 
#
# No authentication might work in an environment where there are very
# few Mon users and they can all be trusted equally, or if you want
# to use mon.cgi in a sort of "read-only" capacity, where all users
# can list, for example, but no web users can enable/disable 
# monitoring and/or control the server in any way.
#
# Alternatively, if you want to use authentication, you need to have
# a working authentication setup with Mon from the command line
# before attempting to make authentication work with mon.cgi.
#
# Authentication is very flexible, and is trivial to implement in 
# mon.cgi, assuming you already have authentication working from 
# the command line. Just un-comment out the "$must_login line, change
# $app_secret to be something unique (VERY IMPORTANT!) and 
# mon will start requiring authentication for ALL commands.
#
# Authentication users should change their app secret on a regular
# basis if security is a concern. Actually, if security is a concern,
# don't run mon.cgi, because unless you use SSL, AND your monhost is
# on the same server as your web server, AND you use a short timeout
# on cookies, AND you change your app secret often and keep it
# VERY secure, you don't have a secure web system. But this simple 
# authentication mechanism is enough to keep most people happy.
#
#
# This script will require the CGI perl module. Available at any
# perl CPAN site. See http://www.perl.org for details. Oh, and don't
# forget Mon::Client, but the assumption is you are already running
# mon in some fashion and so you know this already.
#
# In addition, if you want to use the authentication piece of mon.cgi,
# you need to install the Crypt::TripleDES module, also available
# (tested w/ Crypt::TripleDES v0.24), and your browser needs to allow
# cookies (or else you need to be prepared to type in your username
# and password an awful lot!).
#
#
# BUGS
#  Probably many.
#  Send bugs/comments about this software to 
#  Andrew Ryan <andrewr@nam-shub.com>
#  Please include any output from your web server's error log that the
#  script might have output, this will help immensely in solving problems.
#  Also please include the versions of mon.cgi, Mon and Mon::Client you 
#  are using.
#


BEGIN {
	# Auto-detect if we are running under mod_perl or CGI.
	$USE_MOD_PERL = exists $ENV{'MOD_PERL'}
	? 1 : 0;
	if ($USE_MOD_PERL) {
		# Use the cgi module and compile all methods at
		# the beginning but only once
		use CGI qw (-compile :standard);
	} else {
		# Use the cgi module and compile all methods only
		# when they are invoked via the autoloader.
		# use CGI qw (-debug) ; #DEBUG
		use CGI qw (:standard) ;
	}
	$CGI::POST_MAX=1024 * 100;  # max 100K posts
	$CGI::DISABLE_UPLOADS = 1;  # no uploads
}


use Mon::Client;			# mon client interface
use strict;				# because strict is GOOD
# NPG
# Try to get all of my init changes into a single spot.
use vars qw( $base_link $stylesheet);	# Vars v1
use vars qw( @AllowedHosts);		# Vars v4
use vars qw( $base_url $cmd_url);	# Vars v5
					# Vars v8
use vars qw( $logo_pos %log_cmds %opstat_cmds $ListCmd $ListLock $SchedCtl $SchedState);
					# Vars v9
use vars qw( $USE_MOD_PERL $ME $PATH $headers_sent @ListCmds @AllowedPorts);
use vars qw( $Toggle $SchedLock $TermLock $TermCmd );	# Vars v10
$USE_MOD_PERL = exists $ENV{'MOD_PERL'} ? 1 : 0;
$base_url = CGI::script_name();			# URL of this script.
($ME = $0) =~ s-.*/--;
($PATH = $base_url) =~s/$ME//;
@ListCmds = qw ( list list_alerthist list_dtlog list_pids mon_test_service query_group query_opstatus svc_details );
# NPG End

# Configurable Parameters ----------------------------------------------
# Basic global vars
use vars qw($RCSID $RCSVERSION $VERSION $AUTHOR $organization $monadmin 
	$logo $logo_link $reload_time $monhost $monport $url
	$login_expire_time $cookie_name $cookie_path %cgiparams
	$vcookie_name $vcookie_path $vcookie @views $curview
	$monhost_and_port_args $has_read_config $moncgi_config_file
	$cf_file_mtime $untaint_ack_msgs @show_watch $show_watch_strict
	$required_mon_client_version);
# Formatting-related global vars
use vars qw($BGCOLOR $TEXTCOLOR $LINKCOLOR $VLINKCOLOR 
	$greenlight_color $redlight_color $unchecked_color 
	$yellowlight_color $disabled_color 
	$fixed_font_face $sans_serif_font_face 
	$dtlog_max_failures_per_page);
# Security-related global vars
use vars qw($must_login $app_secret %loginhash $des $has_prompted_for_auth 
	$destroy_auth_cookie $default_username $default_password);
$has_prompted_for_auth = "";	# this must always be cleared for mod_perl
undef $destroy_auth_cookie;	# this must always be undef'd for mod_perl
undef %cgiparams;		# this must always be undef'd for mod_perl
undef $monhost_and_port_args;	# This is defined if the user overrided monhost or monport
undef @show_watch;
undef $show_watch_strict;

$RCSID = '$Id: mon.cgi,v 1.4.2.1 2007/05/03 19:55:38 trockij Exp $';
# NPG
$RCSVERSION = '$Revision: 1.4.2.1-CMRv11 $';
$AUTHOR = 'nathan@cmpublishers.com';
# NPG end
$VERSION = $RCSVERSION;
$VERSION =~ s/\Revision: //i; $VERSION =~ s/\$//g ; $VERSION =~ s/\s+//g;
$required_mon_client_version = "0.11";	# Version of Mon::Client which we require for successful operation

# If you want to use a config file to specify mon.cgi parameters, put the full 
# path to the file in this variable.  Otherwise leave blank.
$moncgi_config_file = "";

# This subroutine initializes the configuration variables which can be set 
# here, but also overridden with the optional mon.cgi config file.
#
# We put this subroutine at the top of the code so that users
# can get to it more easily.
#
sub initialize_config_globals ;
sub initialize_config_globals {
	undef $has_read_config ;		# undef this for mod_perl
	$must_login = "";			# this must always be undef'd 
						# for mod_perl
	$organization = "";			# Organization name.
	$monadmin = "";				# Your e-mail address.
						# note: must escape @ signs!
	# NPG
	$base_link = "";			# URL of site mon.cgi is part of
	$logo_link = "";			# URL that logo will pont to
	$logo_pos = "l";			# Logo Pos l or r
						# or if you want to hack the
						# css anything you want.
	$stylesheet = "mon.css";		# CSS File
	$cookie_name = "mon-cookie";		# Cookie given to browser for
						# auth
	$cookie_path = "$PATH";			# path for auth cookie
	$vcookie_name = "monview-cookie";	# Cookie given to browser for
						# view
	$vcookie_path = "$PATH";		# path for view cookie
	# NPG End
	$logo = "";				# Company or mon logo.
	$reload_time = 60;			# Seconds for page reload.
	$monhost = "localhost";			# Mon server hostname.
	$monport = "2583";			# Mon port number.
	$must_login = "no";		# Switch to yes if you want
					# authentication to be mandatory for
					# all connections to the mon server.
					#!!! WARNING!!!!! You must change 
					# $app_secret to something unique to 
					# your site!
	$app_secret = '1.90LK=R==36jlel492jl><><oiOIUIGWgIgiiGYrrEGb9c<udY^dz{z9Vg57(aYt#RLaw4:)y?j0h.!()()1&jjJJJHGIp^YS'; # something unguessable
	$default_username = "readonly";	# default username to try when 
					# authenticating (should be a 
					# low-privilege account)
	$default_password = "public" ;	# default password to try when
					# authenticating (should be a
					# low-privilege account)
	$login_expire_time = 900;	# Idle time, in seconds, until login
					# cookie is invalidated. Note that if
					# $login_expire_time < $reload_time,
					# you will not be able to "idle" and
					# authentication will be required much
					# more often.
	$untaint_ack_msgs = "yes";	# Use HTML::Entities to scrub
					# user-supplied ack messages
					# (recommended!)
	# Define optional regexes in the @show_watch variable,
	# and only hostgroups which match one of these regexes
	# will be shown.
	# @show_watch = ("hostgroup1", "hostgroup2", "www.*");
	$show_watch_strict = "no";	# set this to "yes" to enforce show_watches strictly
	$fixed_font_face = "courier";			#default 'fixed' font-face to use
	$sans_serif_font_face = "Helvetica, Arial";	#default 'sans-serif' font-face to use
	$BGCOLOR = "black";		# Background color
	$TEXTCOLOR = "#D8D8BF";		# Text color (default is gray)
	$LINKCOLOR = "yellow";		# Link color
	$VLINKCOLOR = "#00FFFF";	# Visited link color (default is teal)
	$greenlight_color = "#009900";	# color of "good" status events
	$redlight_color = "red";	# color of "bad" status events
	$unchecked_color = "#000088";	# color of "unknown" status events
	$disabled_color="#999999";	# color of "disabled" items
	$yellowlight_color = "#FF9933";	# color of "going bad" status events
	$dtlog_max_failures_per_page = "100";	# Maximum dtlog entries to show per page
}

###############################################################
# You shouldn't need to change anything below this !!!!!!!!
###############################################################

# General Declarations -------------------------------------------------
use vars qw($c $connect_failed $webpage $time $localtime @year_months @days_of_week );
@year_months = ('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec');
@days_of_week = ('Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday');
$connect_failed = 0;	# we assume the connection will succeed until proven otherwise
%loginhash = ("username","", "password","");
$webpage = new CGI;			# Initialize web page CGI object
$time = time;				# This will be used for the time on the
$localtime = localtime(time);		# page. (should perhaps use servertime?)
use vars  qw($command $args $rt $rtargs $ackcomment %alert_vars 
	@time_based_alert_vars @pp_sec_alert_vars %auth_commands 
	$auth_commands_checked );
# These global vars all need to be zeroed out at the beginning of each
# instance so that they don't confuse mod_perl. If you're running under CGI,
# you don't need to do this, but it won't hurt either.
$command = "";
$ackcomment = "";
$args = "";
$rt = "";
$rtargs = "";

# Initialize global variables, by source and/or an optional config file
# Read the mon.cgi configuration file if one was specified
if ($moncgi_config_file) {	# config file specified
	# First read the configuration global variables, if we haven't yet.
	# But only read in config globals once from the source code, after a
	# config file has been read in, the config file is the authoritative 
	# source of configuration information.
	&initialize_config_globals unless $has_read_config;
	# NPG
	@AllowedHosts = ( @AllowedHosts, $monhost);
	@AllowedPorts = ( @AllowedPorts, $monport);
	# NPG End
	# Then override these with settings from the config file.
	&moncgi_read_cf($moncgi_config_file) if $moncgi_config_file;
} else {	# no config file specified
	# initialize config globals unconditionally
	&initialize_config_globals;
}
# Note that monhost and monport can later be overridden CGI params.

# Check for a proper version of Mon::Client. If the user has a
# version that's too old, tell them so and exit.
if ($Mon::Client::VERSION < $required_mon_client_version) {
	# NPG
	print $webpage->header(-status=>501);
	# NPG End
	print $webpage->start_html(-title=>"mon.cgi error: insufficient Mon::Client version",
		-BGCOLOR=>$BGCOLOR,-TEXT=>$TEXTCOLOR,
		-LINK=>$LINKCOLOR,-VLINK=>$VLINKCOLOR,
		-META=>{'generator'=>"mon.cgi $VERSION ($AUTHOR)",},
	);
	# NPG
	print $webpage->h3("Insufficient version ($Mon::Client::VERSION) of the Mon::Client perl module installed.<br/> Please upgrade your Mon::Client to at least version $required_mon_client_version before running mon.cgi.");
	if ($USE_MOD_PERL) {
		print $webpage->h3("Mon.cgi is running under Apache+mod_perl.<br/>Restart Apache after upgrading the Mon::Client library.");
	}
	# NPG End
	print $webpage->end_html;
	exit;
}

# Read CGI params -- these overwrite anything in a config file or hard-coded.
# This can change the value of the $monhost and $monport global variables 
# defined in initialize_config_globals and moncgi_read_cf.  This can cause a 
# problem if $monhost or $monport are defined here and we are running mod_perl.
&moncgi_get_params;

# NPG
# Now that we have monhost_and_port_args lets encode it into $url if needed.
if ($monhost_and_port_args) {
	$url = $base_url . "?${monhost_and_port_args}";
	$cmd_url = $base_url . "?${monhost_and_port_args}command";
}else{
	$url = $base_url;
	$cmd_url = $base_url . "?command";
}
# Process $rt & $rtargs if needed.
if ( $command eq "" && $rt ne "" ) {
	$command = $rt;
}
if ( $args eq "" && $rtargs ne "" ) {
	$args = $rtargs;
}
# Menu Data
%opstat_cmds = (
	'1Summary' => "summary",
	'2Failures' => "failures",
	'3Full' => "full", 
	'4Disabled' => "disabled",
);
%log_cmds = ( 
	'1Alerts' => "alerthist",
	'2Downtime' => "dtlog",
);
# NPG End

# Used to escape HTML in ack's
if ($untaint_ack_msgs =~ /^y(es)?$/i) {
	eval "use HTML::Entities" ;
} else {
	undef $untaint_ack_msgs;
}

# NPG
# Initialize a TripleDES global if login is required, 
# otherwise undef $must_login
# Also Set (or unset) $show_watch_strict according to its value
if ($must_login =~ /^y(es)?$/i) {
	eval "use Crypt::TripleDES";
	$des = new Crypt::TripleDES;
	$show_watch_strict = 1;
} else {
	$must_login = "";
	$show_watch_strict = "";
}
# NPG End

# Initialize the wordy descriptions of alert variables
%alert_vars = (
	'opstatus' => "Current status of this service (0=error, 1=OK, 7=unchecked)",
	'alerts_sent' => "Number of alerts sent",
	'monitor' => "Monitor used to test this service",
	'last_trap' => "Last time a trap was received on this service",
	'failure_duration' => "Length of failure",
	'ack' => "Acknowledgement status (1=failed service was ack'ed)",
	'ackcomment' => "Comment issued by the acknowledger",
	'service' => "Service being checked",
	'last_opstatus' => "Previous opstatus for this service (0=error, 1=OK, 7=unchecked)",
	'exitval' => "Last exit value of monitor for this service (0=OK, anything else indicates failure)",
	'depstatus' => "Dependency status (1 = dependencies OK, 0=dependencies not OK or no dependencies)",
	# NPG
	'depend' => "Dependencies, if any",
	'group' => "Host Group",
	'interval' => "Test Interval",
	'last_check' => "Last Check",
	'last_success' => "Last OK",
	'last_detail' => "Detail output from the most recent service check",
	'last_summary' => "Summary output from most recent service check",
	'monitor_running' => "Test in Progress",
	'timer' => "Next Check",
	'first_failure' => "Failure first seen",
	'last_failure' => "Failure last seen",
	'last_alert' => "Alert last sent",
	# NPG end
);

# These are variables from svc_details which should be represented as
# pretty-printed time strings. Mon gives them to us as UNIX time(2), so we
# have to convert. They used to be hardcoded deep into the code, this is an 
# attempt at a readability improvement.
# Example representation: '(31 days, 17 hours, 53 minutes, 25 seconds ago)'
@time_based_alert_vars = (
	"last_check","last_failure","last_trap","last_alert",
	"last_success","last_failure","first_failure",
);

# These are variables from svc_details which should be represented as
# pretty-printed seconds/minutes/hours/days.
# Example representation: '4 minutes, 19 seconds'
@pp_sec_alert_vars = ("timer","interval","failure_duration",);

%auth_commands = (
	# This global tracks the authorization status of all commands 
	# mon.cgi can issue for a user. It is a candidate for inclusion in a 
	# cookie someday.
	list =>      { auth=>0, bgcolor=>""},
	reset =>     { auth=>0, bgcolor=>""},
	stop =>      { auth=>0, bgcolor=>""},
	start =>     { auth=>0, bgcolor=>""},
	savestate => { auth=>0, bgcolor=>""},
	loadstate => { auth=>0, bgcolor=>""},
	disable =>   { auth=>0, bgcolor=>""},
	enable =>    { auth=>0, bgcolor=>""},
	test =>      { auth=>0, bgcolor=>""},
	ack =>       { auth=>0, bgcolor=>""},
	reload =>    { auth=>0, bgcolor=>""},
	# NPG
	term =>      { auth=>0, bgcolor=>""},
	# NPG End
);
$auth_commands_checked = 0;	# This global tracks whether authorization for
				# all commands has been checked.

###############################################################
# Function definitions begin below
###############################################################
# Forward declare all functions, for our sanity.
# General functions
sub pp_sec ;
sub pp_sec_brief ;
sub arithmetic_mean ;
sub median ;
sub std_dev ;
# NPG
sub validate_group;
sub validate_gs;
sub TimeStamp;
sub print_status;
# NPG End
sub validate_name ;
sub gen_ciphertext ;
sub gen_cleartext ;
# Base mon.cgi pages
sub setup_page ;
sub print_bar ;
sub query_opstatus ;
sub can_show_group ;
sub list_status ;
sub query_group ;
sub end_page ;
sub list_alerthist ;
sub svc_details ;
sub list_dtlog ;
sub list_pids ;
# mon functions
sub mon_connect ;
# NPG
sub mon_connectfail ;
sub mon_entity_mgr ;
sub mon_list ;
sub mon_state_mgr ;
sub mon_stop ;
# NPG End
sub mon_views;
sub mon_reload ;
sub mon_loadstate ;
sub mon_savestate ;
sub mon_schedctl ;
sub mon_enable ;
sub mon_disable ;
sub mon_test_service ;
sub mon_test_config ;
sub mon_reset ;
sub mon_list_dtlog ;
sub mon_ack ;
sub mon_servertime ;
sub mon_checkauth ;
sub mon_state_change_enable_only ;
sub mon_state_change ;
# mon.cgi functions
sub moncgi_get_params ;
sub moncgi_logout ;
sub moncgi_authform ;
sub moncgi_authfail ;
sub moncgi_generic_button ;
sub moncgi_switch_user ;
sub moncgi_print_service_table_legend ;
sub moncgi_list_dtlog_navtable ;
sub moncgi_test_all ;
sub moncgi_reset ;
# NPG
sub moncgi_stop ;
# NPG End
sub moncgi_read_cf ;
sub moncgi_login ;
sub moncgi_custom_print_bar ;
sub moncgi_custom_commands;

###############################################################
# General functions, not specific to Mon or mon.cgi
###############################################################
sub pp_sec {
	# This routine converts a number of seconds into a text string
	# suitable for (pretty) printing. The dtlog from Mon reports downtime
	# in seconds, and we want to present the user with more meaningful
	# data than "the service has been down for 13638 seconds"
	#
	# By Martha Greenberg <marthag@mit.edu> w/ pedantic plural 
	# modifications by Andrew.
	# Suppression of 0 seconds in output by NPG
	use integer;
	my $n = $_[0];
	my ($days, $hrs, $min, $sec) = ($n / 86400, $n % 86400 / 3600,
	$n % 3600 / 60, $n % 60);
	# NPG
	# Makes second handling code consistent with everything else.
	# Suppresses output when seconds == 0
	my $s;
	if ( $sec > 0) {
		if ($sec == 1) {
			$s = $sec . " second";
		} else {
			$s = $sec . " seconds";
		}
	}
	# NPG End
	if ($min > 0) {
		if ($min == 1) {
			$s = $min . " minute, " . $s;
		} else {
			$s = $min . " minutes, " . $s;
		}
	}
	if ($hrs > 0) {
		if ($hrs == 1) {
			$s = $hrs . " hour, " . $s;
		} else {
			$s = $hrs . " hours, " . $s;
		}
	}
	if ($days > 0) {
		if ($days == 1) {
			$s = $days . " day, " . $s;
		} else {
			$s = $days . " days, " . $s;
		}
	}
	return $s;
}

sub pp_sec_brief {
	# This routine converts a number of seconds into a text string
	# suitable for brief (yet pretty) printing.
	# We use this on the opstatus page to display deltas of times (for
	# last check, next check).
	use integer;
	my $n = $_[0];
	my $s;
	# NPG
	# Suppress signed output
	# As last checks will always be negative, and next checks always 
	# positive,
	# Why clutter up the output with signs.
	if ($n < 0) {
		$n = abs($n);
	}
	# NPG End
	my ($days, $hrs, $min, $sec) = ($n / 86400, $n % 86400 / 3600,
	$n % 3600 / 60, $n % 60);
	if ($days > 0) {
		$s .= $days . "d ";
	}
	if ($hrs > 0) {
		$s .= $hrs . "h ";
	}
	if ($min > 0) {
		$s .= $min . "m ";
	}
	# NPG
	# Makes second handling code consistent with everything else.
	# Suppresses output when seconds == 0
	if ($sec > 0) {
		$s .= "${sec}s" ;
	}
	# When all else failes return something. This fixes some transient 
	# output errors when the logic in list_status didn't catch it.
	if ( $s eq "" ) {
		$s = "--";
	}
	# NPG End
	return $s;
}

sub arithmetic_mean {
	# Given an array of numbers, this function returns the arithmetic mean
	return 0 if scalar(@_) == 0 ;	# don't waste our time
	my $sum = 0;
	foreach (@_) {
		$sum += $_;
	}
	return $sum/scalar(@_);
}

sub median {
	# Given an array of numbers, this function returns the median
	return 0 if scalar(@_) == 0 ;	# don't waste our time
	my $middle_element_index = int(scalar(@_)/2);
	@_ = sort {$a <=> $b} @_;
	if (scalar(@_)%2 == 1) {	# odd num of elements, take the middle
		return $_[$middle_element_index];
	} else {	# even # of elements, take the avg of the 2 middle elements
		return &arithmetic_mean($_[$middle_element_index],$_[($middle_element_index-1)]);
	}
}

sub std_dev {
	# Given an array of numbers, this function returns their standard
	# deviation.
	return 0 if scalar(@_) < 2 ;	# don't waste our time
	my $sum = 0;
	my $mean = &arithmetic_mean(@_);
	foreach (@_) {
		$sum += ( $_ - $mean )**2 ;
	}
	return ( $sum/(scalar(@_) - 1) )**0.5 ;
}

# NPG
# After running the "XSS ME" FF Extension.
# Against mon.cgi it became appearant that some input validation was in order.
sub validate_group {
	# Validate a group.
	my ($group) = (@_);
	my (%op, $Ok);
	if ( $ListLock ) {
		# We cant even find out, don't trust the data.
		$Ok = -1;
	}else{
		%op = &mon_list("opstatus", "$group");
		if ($op{$group}) {
			$Ok = 1;
		} else {
			# Its not in the mon process, don't trust the data.
			$Ok = 0;
		}
	}
	return $Ok;
}
sub validate_gs {
	# Validate a group & service pair.
	my ($group, $service) = (@_);
	my (%op, $status);
	if ( $ListLock ) {
		# We cant even find out, don't trust the data.
		$status = -1;
	}else{
		%op = &mon_list("opstatus", "$group,$service");
		if ($op{$group}{$service}) {
			$status = 1;
		} else {
			# Its not in the mon process, don't trust the data.
			$status = 0;
		}
	}
	return $status;
}
# A Standardized Time Stamp.
sub TimeStamp {
	my $Type = shift;
	my $localtime = shift;
	my @time = localtime($localtime);
	if ( $Type eq "ST" || $Type eq "" ) {
		$localtime = sprintf ("%s %.d %d, %.2d:%.2d:%.2d", @year_months[$time[4]], $time[3], $time[5] + 1900, @time[2,1,0] );
	}elsif ($Type eq "TD") {
		$localtime = sprintf ("%s %.d %d<br/>%.2d:%.2d:%.2d", @year_months[$time[4]], $time[3], $time[5] + 1900, @time[2,1,0] );
	}elsif ($Type eq "Cookie") {
		@time = gmtime($localtime);
		$localtime = sprintf ("%s, %.2d-%s-%d %.2d:%.2d:%.2d GMT", @days_of_week[$time[6]], $time[3], @year_months[$time[4]], $time[5] + 1900, @time[2,1,0]);
	}else{
		$localtime = "Invalid";
	}
	return $localtime;
}
# NPG End

sub validate_name {
	# Return untainted host or group name if safe, undef otherwise.
	# Because you can never scrub your input too well.
	return $_[0] =~ /^([\w.\-_]+)$/ ? $1 : undef;
}

sub gen_ciphertext {
	# This function takes as its input a piece of plaintext and returns
	# a piece of ASCII, 3DES-encoded ciphertext, or undef if 3DES fails
	# for some reason. The key used is the global value "$app_secret".
	my ($plaintext) = (@_);
	my $ciphertext ;
	if ($ciphertext = $des->encrypt3 ("$plaintext", "$app_secret" )) {
		# convert key to hex
		$ciphertext = unpack("H*", $ciphertext) ;
		return $ciphertext;
	} else {
		return undef;
	}
}

sub gen_cleartext {
	# This function takes as its input a piece of ASCII, 3DES-encoded
	# ciphertext and returns a piece of plaintext, or undef if 3DES fails
	# for some reason. The key used is the global value "$app_secret".
	my ($ciphertext) = (@_);
	my $plaintext ;
	return undef if ! $ciphertext;
	# convert key to format decrypt3() will understand
	$ciphertext = pack("H*", $ciphertext) ;
	if ($plaintext = $des->decrypt3 ("$ciphertext", "$app_secret" )) {
		return $plaintext;
	} else {
		return undef;
	}
}

###############################################################
# Presentation functions. These all have the common feature
# that they format a bunch of information and present it in 
# a nice(?) way to the user.
###############################################################
sub setup_page {
	# Setup the html doc headers and such
	# Also, get/set username/password cookie if $must_login is in effect
	# NPG
	my ($title,$http_status ) = (@_);
	# NPG End
	my $title_color = "$TEXTCOLOR"; 
	my $page_title = "$organization : " if $organization ne "";	
	my $time_now = time;
	# Put the cookie date format in the standard cookie format
	my $expires = &TimeStamp ("Cookie", ($time_now + $login_expire_time));
	my ($encrypted_password, $cookie, $cookie_value);
	my $refresh_url;
	# Define $args as null if it is not currently defined
	$args = "" if ! defined($args);
	# Set the refresh page to always be the summary page, unless certain 
	# NPG
	if (!$http_status){
		$http_status = 200;
	}
	if ( !$headers_sent){
		if ($http_status == 200){
			# Catch Connection Errors
			if ($connect_failed) {
				$http_status = 502;
				$title = "Connection Error.";
			}else{
				if ( 
					( $ListLock == 1 && $ListCmd == 1 ) || 
					( $TermLock == 1 && $TermCmd == 1 )
				) {
					$http_status = 403;
					$title = "Auth Error.";
				}
			}
			if ($http_status == 200){
				# Set up Refresh URL.
				if ( $command eq "query_opstatus" ) {
					$refresh_url = "$cmd_url=$command&args=$args";
	# NPG End
				} elsif ( ($command eq "mon_test_service") || ($command eq "svc_details") ) {
					# NPG
					$refresh_url = "$cmd_url=svc_details&args=$args";
					# NPG End
				} elsif ($command eq "query_group") {
					# NPG
					$refresh_url = "$cmd_url=query_group&args=$args";
					# NPG End
				} elsif ($command eq "list_dtlog") {
				# NPG
					$refresh_url = "$cmd_url=list_dtlog&args=$args";
				} elsif ($command eq "list_alerthist") {
					$refresh_url = "$cmd_url=list_alerthist&args=$args";
				} elsif ($command eq "list_pids") {
					$refresh_url = "$cmd_url=list_pids";
				} elsif ($command eq "list") {
					$refresh_url = "$cmd_url=list&args=$args";
				} elsif ($command eq "mon_state_change") {
					if ($rt eq "query_group") {
						$refresh_url = "$cmd_url=$rt&args=$rtargs";
					}else{
						$refresh_url = "$url";
					}
				} elsif ( $command eq "mon_disable" || $command eq "mon_enable" ) {
					if ($rt eq "svc_details") {
						$refresh_url = "$cmd_url=$rt&args=$rtargs";
					}else{
						$refresh_url = "$url";
					}
				}elsif ( $command eq "moncgi_reset" || $command eq "mon_reset") {
					$refresh_url = "$cmd_url=moncgi_reset";
				}elsif ($command eq "mon_schedctl") {
					if ($rt eq "list_pids") {
						$refresh_url = "$cmd_url=$rt&args=$rtargs";
					}else{
						$refresh_url = "$url";
					}
				# NPG End
				} else {
				# NPG
					# Shortened this as the main sections sets up the default cmd.
					$refresh_url = "$url";
				# NPG End
				}
				# NPG
				# Setup Cookies
				# NPG End
				if ($must_login) {
					if ( ( defined($loginhash{'username'}) ) &&
					( $loginhash{'username'} ne "" ) ) {
						# Don't get the username and
						# password from the cookie if
						# the user just submitted it
						# via the login form.
						# Encrypt the password for
						# cookie storage.
						$encrypted_password = &gen_ciphertext($loginhash{'password'}) ;
					} else {
						# Get the existing cookie and
						# parse it
						$cookie_value = $webpage->cookie(-name=>"$cookie_name",);
						($loginhash{'username'},$loginhash{'password'}) = split(':', $cookie_value) if $cookie_value;
						$encrypted_password = $loginhash{'password'} ;
						# Decrypt the password string
						# (if any) for use by the app,
						# unless the user just
						# submitted it in cleartext.
						$loginhash{'password'} = &gen_cleartext($loginhash{'password'}) ;
						# for some reason (bug?) I get
						# a space at the end of the
						# password that is returned
						# here, so for now let's take
						# it out, since spaces are
						# illegal in passwords anyway.
						$loginhash{'password'} =~ s/\s+//g if defined($loginhash{'password'}) ;
					}
					# Set up the new cookie (re-issue a
					# new cookie with every access)
					if ($destroy_auth_cookie) {
						$cookie_value = "" ;
					} elsif ( defined($loginhash{'password'}) ) {
						$cookie_value = "$loginhash{'username'}:$encrypted_password" ;
					} else {	# no username was supplied
						$cookie_value = "" ;
					}
					$cookie = $webpage->cookie(-name=>"$cookie_name",
						-value=>"$cookie_value",-expires=>"$expires",
						# NPG
						-path=>"$cookie_path",-httponly =>'1',
						# NPG End
					);
					print $webpage->header(-cookie=>[$cookie,$vcookie],
						-refresh=>"$reload_time; URL=$refresh_url",
						# NPG
						-status=>$http_status,
						# NPG End
					);
				} else {
					# Plain & simple, no cookie, no passsword
					$encrypted_password = "" ;
					print $webpage->header(-cookie => [$vcookie],
						-refresh=>"$reload_time; URL=$refresh_url",
						# NPG
						-status=>$http_status,
						# NPG End
					);
				}
		# NPG
			}else{
				print $webpage->header(-status=>$http_status);
			}
		}elsif ($http_status == 403){
			print $webpage->header(-status=>$http_status);
			$title = "Auth Error.";
		}elsif ($http_status == 502){
			print $webpage->header(-status=>$http_status);
			$title = "Connection Error.";
		}else{
			# Caller set HTTP Status
			print $webpage->header(-status=>$http_status);
			$title = "Other Error";
		}
		$headers_sent = 1;
		# Removed $monhost & $monport from page title.
		# Unnecessary information disclosure.
		$page_title = "${page_title}MON - $title";
		# NPG End
		print $webpage->start_html(-title=>"$page_title",
			-BGCOLOR=>$BGCOLOR,-TEXT=>$TEXTCOLOR,
			-LINK=>$LINKCOLOR,-VLINK=>$VLINKCOLOR,
			-META=>{ 'generator'=>"mon.cgi $VERSION ($AUTHOR)", },
			# NPG
			# Add css support
			-style=>{ 'src'=>"$base_link/$stylesheet", 'media'=> "all"},
			# NPG End
		);

# Useful for debugging username/password/cookie issues  #DEBUG
#    print "cookie value is $cookie_value<br>\n";  #DEBUG
#    print "encrypt passwd is $encrypted_password<br>\n";  #DEBUG
#    print "username is &quot;$loginhash{'username'}&quot;<br>\n";  #DEBUG
#    print "decrypt passwd is &quot;$loginhash{'password'}&quot;<br>\n";  #DEBUG

		# NPG
		# Print the logo image, if defined by the user.
		# Provide a Link leading out of mon.cgi if one is defined.
		# Replaced the top table with <div> <span> & some CSS
		if ($logo) {
			$webpage->print("<div class=\"pagetop\">");
			# If the user has given a logo_link, then insert an
			# anchor tag to it here.
			if ($logo_link) {
				$webpage->print("<a href=\"$logo_link\"><img class=\"logo$logo_pos\" src=\"$base_link/$logo\" alt=\"[$organization logo]\"/></a>");
			} else {
				$webpage->print("<img class=\"logo$logo_pos\" src=\"$base_link/$logo\" alt=\"[$organization logo]\"/>");
			}
		} else {
			$webpage->print("<div>");
			if ($base_link) {
				$webpage->print("<a href=\"$base_link\">Home</a>");
			}
		}
		# just print the generic page with no logo and link
		$webpage->print("<span class=\"header\">MON: $title</span>");
		$webpage->print("</div>\n");
		print $webpage->hr;
		if ($http_status == 200 ){
		# NPG End
			&print_bar;
		# NPG
		}
		# Change DateStamp Format.
		my $ttime = &TimeStamp("ST", $time);
		# NPG End
		$webpage->print("<center>\n");
		$webpage->print("This information was presented on $ttime");
		# NPG
		# If the user is currently logged in, offer to log them out.
		# If the user is NOT currently logged in, offer to log them in.
		if (!$connect_failed && $must_login) {
		# NPG End
			if ( ($loginhash{'username'}) && 
			($loginhash{'username'} ne $default_username) ) {
				# NPG
				$webpage->print
				(" (<a href=\"$cmd_url=moncgi_logout\">log out</a>)");
				# NPG End
			} else {
				$webpage->print
				(" (<a href=\"$cmd_url=moncgi_login\">log in</a>)");
			}
	# NPG
		}
		if ($http_status == 200){
			$webpage->print(".<br/>This page will reload every $reload_time seconds.<br/>");
		}elsif($http_status == 403) {
			$webpage->print(".<br/>Auth Error!<br/>\n");
		}
		$webpage->print("</center>");
	}
	# NPG End
}

sub print_bar {
	# Print the command bar. Called at the beginning of each page.
	my $button = "INPUT TYPE=\"submit\" NAME=\"command\"";
	my $face = $sans_serif_font_face;
	my ($cmd, $command, $desc);
	my $all_commands_unauthorized = 1;
	my $i;
	# NPG
	# If we can't connect to the server the command bar will be useless, so
	# don't show it.
	if (!$connect_failed) {
		foreach $cmd (keys %auth_commands) {
			$all_commands_unauthorized = 0 if $auth_commands{$cmd}{'auth'} == 1 ;
		}
		# Check to see if authentication is required and the selected user has
		# no permissions to do anything. This can only happen if your mon admin
		# is very cruel and gives you an account with no permissions (should
		# be very rare), or if a user enters the wrong password for a valid
		# account name (the usual case).
		# The main thing I don't like about this solution is that it embeds
		# the default l/p into the URL, but this isn't a secure password
		# anyway, right?
		$cmd = "list";	# set $cmd to the most basic of commands
		if ($auth_commands{$cmd}{'auth'} == -1 ) {
			$webpage->print("<h2>Cannot connect to the mon server. Check the mon process to see if it is running.</h2><br/>\n");
			$webpage->print("<h2>" . &mon_connectfail . "</h2><br/>\n");
		} elsif ($auth_commands{$cmd}{'auth'} == 0 ) {
			$webpage->print("<h2>Auth Error. (perhaps you don't have permissions in auth.cf?).</h2><br/>\n");
		} elsif ( ($must_login) && ($all_commands_unauthorized == 1) ) {
			$webpage->print("<h2>You are attempting to log in with the username &quot;$loginhash{'username'}&quot; but your password is incorrect.<br/><br/>Either enter in the correct username/password above or <a href=\"$cmd_url=query_opstatus&amp;username=$default_username&amp;password=$default_password\">click here to clear your authentication credentials and log back in as the default user.</h2><br/>\n");
		}
		$auth_commands_checked = 1;
		# Table Width & alignment defined in CSS
		# Print a Header for the command table
		# Make Command Links less verbose. Take advantage of new header.
		# We have to lay the tables out by hand bec. of the colspanning :(
		$webpage->print("<table class=\"bar\" summary=\"MonCGI Command Bar\">\n");
		$webpage->print("<tr>\n");
		$webpage->print("\t<th><font size=\"+1\" color=\"$TEXTCOLOR\" face=\"$face\">Operational Status</font></th>\n");
		$webpage->print("\t<th><font size=\"+1\" color=\"$TEXTCOLOR\" face=\"$face\">Logs</font></th>\n");
		$webpage->print("\t<th><font size=\"+1\" color=\"$TEXTCOLOR\" face=\"$face\">Control Panel</font></th>\n");
		$webpage->print("</tr>\n");
		# Print the command table
		$webpage->print("<tr>\n");
		$webpage->print("<td $auth_commands{'list'}{'bgcolor'}>");
		if ( $auth_commands{'list'}{'auth'} == 0 ) {
			$webpage->print("Auth Error");
			$webpage->print("</td>\n");
			$webpage->print("\t<td $auth_commands{'list'}{'bgcolor'}>");
			$webpage->print("Auth Error");
		}elsif ( $auth_commands{'list'}{'auth'} == -1 ) {
			$webpage->print("Connect Error");
			$webpage->print("</td>\n");
			$webpage->print("\t<td $auth_commands{'list'}{'bgcolor'}>");
			$webpage->print("Connect Error");
		}else{
			my @opstats;
			my %Label;
			foreach my $tmp ( sort keys %opstat_cmds) {
				my $tm = substr ($tmp, 1);
				push ( @opstats, "$opstat_cmds{$tmp}");
				$Label{$opstat_cmds{$tmp}} = $tm;
			}
			print $webpage->start_form;
			$webpage->param('command','query_opstatus');
			print $webpage->popup_menu(
			-name=>'args', -values=>[@opstats], -labels=>\%Label,
			);
			print $webpage->hidden(-name=>'command',);
			print $webpage->hidden(-name=>'args',);
			print $webpage->submit(-name=>'Change');
			print $webpage->end_form;
			$webpage->print("</td>\n");
			$webpage->print("\t<td $auth_commands{'list'}{'bgcolor'}>");
			my @logs;
			%Label;
			foreach my $tmp ( sort keys %log_cmds) {
				my $tm = substr ($tmp, 1);
				push ( @logs, "$log_cmds{$tmp}");
				$Label{$log_cmds{$tmp}} = $tm;
				my $tm = substr ($tmp, 1);
			}
			print $webpage->start_form;
			$webpage->param('command',"list");
			print $webpage->popup_menu(
			-name=>'args', -values=>[@logs], -labels=>\%Label,
			);
			print $webpage->hidden(-name=>'command',);
			print $webpage->hidden(-name=>'args',);
			print $webpage->submit(-name=>'View');
			print $webpage->end_form;
		}
		$webpage->print("</td>\n");
		$webpage->print("<td width=\"40%\"><table summary=\"Control Panel\"><tr>\n");
		$webpage->print("\t<td $auth_commands{'test'}{'bgcolor'}>");
		if ( $auth_commands{'test'}{'auth'} == 0 ) {
			$webpage->print("Auth Error");
		}elsif ( $auth_commands{'test'}{'auth'} == -1 ) {
			$webpage->print("Connect Error");
		}else{
			print $webpage->start_form;
			$webpage->param('command',"mon_test_config");
			print $webpage->hidden(-name=>'command',);
			print $webpage->submit(-name=>'Test Config');
			print $webpage->end_form;
		}
		$webpage->print("</td>\n");
		$webpage->print("\t<td $auth_commands{'reload'}{'bgcolor'}>");
		if ( $auth_commands{'reload'}{'auth'} == 0 ) {
			$webpage->print("Auth Error");
		}elsif ( $auth_commands{'reload'}{'auth'} == -1 ) {
			$webpage->print("Connect Error");
		}else{
			print $webpage->start_form;
			$webpage->param('command',"mon_reload");
			$webpage->param('args',"auth");
			print $webpage->hidden(-name=>'command',);
			print $webpage->hidden(-name=>'args',);
			print $webpage->submit(-name=>'Reload Auth');
			print $webpage->end_form;
		}
		$webpage->print("</td>\n");
		$webpage->print("\t<td $auth_commands{'list'}{'bgcolor'}>");
		if ( $auth_commands{'list'}{'auth'} == 0 ) {
			$webpage->print("Auth Error");
		}elsif ( $auth_commands{'list'}{'auth'} == -1 ) {
			$webpage->print("Connect Error");
		}else{
			print $webpage->start_form;
			$webpage->param('command',"list_pids");
			print $webpage->hidden(-name=>'command',);
			print $webpage->submit(-name=>'Process Info');
			print $webpage->end_form;
		}
		$webpage->print("</td>\n");
		$webpage->print("\t<td $auth_commands{'reset'}{'bgcolor'}>");
		if ( $auth_commands{'reset'}{'auth'} == 0 ) {
			$webpage->print("Auth Error");
		}elsif ( $auth_commands{'reset'}{'auth'} == -1 ) {
			$webpage->print("Connect Error");
		}else{
			print $webpage->start_form;
			$webpage->param('command',"moncgi_reset");
			print $webpage->hidden(-name=>'command',);
			print $webpage->submit(-name=>'Reset Mon');
			print $webpage->end_form;
		}
		$webpage->print("</td>\n");
		$webpage->print("</tr>\n");
		$webpage->print("</table></td>\n");
		$webpage->print("</tr>\n");
		# Print the optional third row of the command table.  This row can be 
		# defined at individual sites and contain commands of your choice. 
		# Thanks to Ed Ravin (eravin@panix.com) for this.
		&moncgi_custom_print_bar($face);	# row 3, if any, for the local site
		$webpage->print("</table><hr/>");
	}
	# NPG End
}
# query the server operational status
sub query_opstatus {
	my ($detail_level) = (@_);
	my ($retval);				# some variables for failures
	my (%op_success, %op_failure, %op_disabled);
	my (@disabled_groups, @disabled_hosts, @disabled_svcs);
	my ($group, $service, $host, $watch);
	# NPG
	my $svc_column_width = "35%";	# Service column width
	my $hg_column_width = "15%";	# Host Group column width
	my @scheduler_status;
	my $Sched_Status;	# Scheduler Status Msg.
	my $Sched_Status_Color;	# Scheduler Status Color
	my $Sched_Toggle_Cmd;	# Scheduler Toggle CMD
	my $enable_command_bgcolor = $auth_commands{'enable'}{'bgcolor'};
	my $Auth_Fail;		# Auth Failure
	my @tests = qw ( summary full failures disabled );
	# Check for appropriate detail values.
	if (!grep /^$detail_level$/, @tests) {
		&setup_page("Operation Status: Error!", 400);
	}elsif ( $connect_failed ) {
		&setup_page("", 502);
	}elsif ( $ListLock ) {
		&setup_page("", 403);
		$Auth_Fail = 1;
	}else{
		if ( $detail_level eq "disabled" || $detail_level eq "failures" ) {
			&setup_page("Operation Status: " . ucfirst($detail_level) . " Only View", 200);
		}else{
			&setup_page("Operation Status: " . ucfirst($detail_level) . " View", 200);
		}
	}
	if ( !$connect_failed ) {
		# Put as much stuff that depends on a good connection below.
		if (!$Auth_Fail) {
			@scheduler_status = &mon_list("scheduler state","") ;
			if ( $detail_level ne "disabled") {
	# NPG End
				%op_success = &mon_list("successes","");
				%op_failure = &mon_list("failures","") ;
		# NPG
			}else{
				%op_disabled = &mon_list("disabled","");
				for (keys %{$op_disabled{"watches"}}) {
					push(@disabled_groups,"<tr><td><font size=\"+1\">$_</font></td>");
					push(@disabled_groups,"<td></td>");
					push(@disabled_groups,"<td bgcolor=\"$enable_command_bgcolor\"><input TYPE=\"checkbox\" VALUE=\"$_\" NAME=enagroup_$_/></td></tr>");
				}
				my @seen_hosts;
				foreach $group (keys %{$op_disabled{"hosts"}}) {
					foreach $host (keys %{$op_disabled{"hosts"}{$group}}) {
						my $host_seen = 0;
						foreach my $x ( @seen_hosts ) {
							if ( $x eq $host ) {
								$host_seen = 1;
							}
						}
						if ( $host_seen ) {
							next;
						}else{
							push(@seen_hosts,"$host");
							push(@disabled_hosts,"<tr><td><font size=\"+1\">$group</font></td>");
							push(@disabled_hosts,"<td><font size=\"+1\">Host -- $host</font></td>");
							push(@disabled_hosts,"<td bgcolor=\"$enable_command_bgcolor\"><input TYPE=\"checkbox\" VALUE=\"$host\" NAME=enahost_$host/></td></tr>");
						}
					}
				}
				foreach $watch (keys %{$op_disabled{"services"}}) {
					foreach $service (keys %{$op_disabled{"services"}{$watch}}) {
						push(@disabled_svcs, "<tr><td><font size=\"+1\">$watch</font></td>");
						push(@disabled_svcs, "<td><font size=\"+1\">Service -- $service</font></td>");
						push(@disabled_svcs,"<td bgcolor=\"$enable_command_bgcolor\"><input TYPE=\"checkbox\" VALUE=\"$watch,$service\" NAME=enasvc_${watch}_$service/></td></tr>");
					}
				}
			}
		}
		# NPG End
		$webpage->print("<a name=\"opstat_tbl\"></a>");
	# NPG
	}
	# Put Scheduler & other runtime details in their own table.
	$webpage->print("<table align=\"center\" class=\"opstatus\" border=\"1\" summary=\"Operation Status Controls\"><tr>\n");
	$webpage->print("<td>\n");
	if ( scalar @AllowedHosts > 1 || scalar @AllowedPorts > 1) {
		# Modified other forms to not set the h & p params.
		# Now the only obvious way to mess with them is here.
		print $webpage->start_form;
		if ( scalar @AllowedHosts > 1) {
			$webpage->print( "Host: ");
			print $webpage->popup_menu(
				-name=>'h', 
				-values=>[sort(@AllowedHosts)],
				-default=>"$monhost",
			);
		}else{
			$webpage->print("$monhost");
		}
		$webpage->print( " : ");
		if ( scalar @AllowedPorts > 1) {
			print $webpage->popup_menu(
				-name=>'p', 
				-values=>[sort(@AllowedPorts)],
				-default=>$monport,
			);
		}else{
			$webpage->print("$monport");
		}
		print $webpage->submit(-name=>'Change');
		print $webpage->end_form;
	}else{
		$webpage->print("$monhost : $monport");
	}
	$webpage->print("</td>\n");
	# Moved this code here because, unless we are querying the
	# opstatus, the view is irrelevent.
	if (@views) {
		$webpage->print("<td width=\"25%\">\n");
		$webpage->print( $webpage->start_form,"Current View: ",
		$webpage->popup_menu(-name=>'setview', -values=>["--all--",
		sort(@views)], -default=>$curview),
		$webpage->submit(-name=>'Change'),$webpage->end_form);
		$webpage->print("</td>\n");
	}
	# Scheduler Control
	if ($ListLock) {
		$Sched_Status_Color = $disabled_color;
		$Sched_Status .= "Auth Error.";
		$Sched_Toggle_Cmd = "";
	}else{
		# Now we deal with the scheduler once.
		$Sched_Status = "Scheduler ";
		if ( !$SchedState || $connect_failed) {
			# Give extra notification if the scheduler is down
			# (this is a big deal!)
			$Sched_Status_Color = $redlight_color;
			if ($connect_failed) {
				$Sched_Status .= "unreachable.";
				$Sched_Toggle_Cmd = "";
			}else{
				if ($scheduler_status[1] != 0) {
					my $pretty_sched_down_time = &TimeStamp("ST",$scheduler_status[1]);
					$Sched_Status .= "stopped on <br/>$pretty_sched_down_time.";
				}else{
					$Sched_Status .= "has just gone down.";
				}
				$Sched_Toggle_Cmd = "start";
			}
		}else{
			$Sched_Status_Color = $greenlight_color;
			$Sched_Status .= "is running.";
			$Sched_Toggle_Cmd = "stop";
		}
	}
	$webpage->print("<td width=\"25%\">\n");
	$webpage->print("<table summary=\"Scheduler Control\"><tr><td>Status</td>\n");
	$webpage->print("<td align=\"center\" bgcolor=\"$Sched_Status_Color\" colspan=\"3\"><font size=\"+1\">$Sched_Status</font></td>");
	if ( !$ListLock && ( $SchedCtl || $SchedLock )) {
		# Formalize Scheduler Control
		$webpage->print("</tr><tr><td>Control</td>");
		if ( $SchedCtl && $Sched_Toggle_Cmd ) {
			$webpage->print("<td>\n");
			print $webpage->start_form;
			$webpage->param('command','mon_schedctl');
			$webpage->param('args',"$Sched_Toggle_Cmd");
			print $webpage->hidden(-name=>'command',);
			print $webpage->hidden(-name=>'args',);
			print $webpage->submit(-name=>"Toggle");
			print $webpage->end_form;
			$webpage->print("</td>\n");
		}
		if ( $SchedLock ) {
			$webpage->print("<td>\n");
			print $webpage->start_form;
			$webpage->param('command','mon_loadstate');
			$webpage->param('args',"");
			print $webpage->hidden(-name=>'command',);
			print $webpage->hidden(-name=>'args',);
			print $webpage->submit(-name=>"Load");
			print $webpage->end_form;
			$webpage->print("</td>\n");
			$webpage->print("<td>\n");
			print $webpage->start_form;
			$webpage->param('command','mon_savestate');
			$webpage->param('args',"");
			print $webpage->hidden(-name=>'command',);
			print $webpage->hidden(-name=>'args',);
			print $webpage->submit(-name=>"Save");
			print $webpage->end_form;
			$webpage->print("</td>\n");
		}
	}
	$webpage->print("</tr></table>\n");
	# End Scheduler Control
	$webpage->print("</td>\n");
	if ( !$SchedState && $Sched_Toggle_Cmd ) {
		$webpage->print("</tr><tr><td align=\"center\" colspan=\"3\" bgcolor=\"$redlight_color\">RESULTS SHOWN BELOW MAY NOT BE CORRECT !</td>");
	}
	$webpage->print("</tr></table>\n");
	# No Connection, no table.
	if ( !$connect_failed ) {
		if (!$Auth_Fail) {
			if ( $detail_level eq "disabled"
			&& ( scalar(@disabled_groups) > 0
			|| scalar(@disabled_hosts) > 0
			|| scalar(@disabled_svcs) > 0 )) {
				# Start the form and set up our defaults
				print $webpage->start_form(-action=>"$url");
				$webpage->param('command','mon_state_change_enable_only');
				print $webpage->hidden(-name=>'command',);
			}else{
				$webpage->print("<a name=\"opstatus_tbl\"></a>");
			}
			$webpage->print("<table align=\"center\" class=\"opstatus\" border=\"1\" summary=\"Operation Status\">\n");
			# Custom header for opstatus_disabled. Also, added
			# Header for the Test Service Column
			$webpage->print("<tr><th width=\"$hg_column_width\"><font size=\"+2\">Host Group</font></th>
			<th width=\"$svc_column_width\"><font size=\"+2\">");
			if ( $detail_level eq "disabled") {
				$webpage->print("Object</font></th>\n");
				$webpage->print("<th><font size=\"+2\">Enable");
			}else{
				$webpage->print("Service</font>&nbsp;<sup><font size=\"-1\"><a href=\"$url#legend\">(legend)</a></font></sup></th>\n");
				$webpage->print("<th><font size=\"+2\">Last Checked</font></th>
				<th><font size=\"+2\">Est. Next Check</font></th>
				<th><font size=\"+2\">Test Service");
			}
			$webpage->print("</font></th></tr>\n");
			# Make disabled actually work.
			if ( $detail_level eq "disabled") {
				# If nothing is disabled, don't show the form.
				if ( scalar(@disabled_groups) > 0
				|| scalar(@disabled_hosts) > 0
				|| scalar(@disabled_svcs) > 0 ) {
					if (scalar(@disabled_groups) > 0 ) {
						print join("\n", @disabled_groups);
					}
					# Disabled hosts portion of table
					if (scalar(@disabled_hosts) > 0 ) {
						print join("\n", @disabled_hosts);
					}
					# Disabled services portion of table
					if (scalar(@disabled_svcs) > 0 ) {
						print join("\n", @disabled_svcs);
					}
					$webpage->print("<tr><td colspan=2 align=\"center\">");
					print $webpage->reset(-name=>'Cancel Changes');
					$webpage->print("</td><td align=\"center\">");
					print $webpage->submit(-name=>'Apply Changes');
					$webpage->print("</td></tr>");
				}else{
					$webpage->print("<tr><td bgcolor=\"$greenlight_color\" colspan=3 align=\"center\"><font size=\"+2\">Nothing disabled.</font></td></tr>");
				}
			}else{
				# Not as concise, but easier to follow than original code.
				if ( %op_failure ) {
					&list_status($detail_level, %op_failure);
				}
				if ( $detail_level eq "failures") {
					if ( !%op_failure ) {
						$webpage->print
						("<tr><td bgcolor=\"$greenlight_color\" colspan=5 align=\"center\"><font size=\"+2\">No failures found.</font></td></tr>\n");
					}
				}else{
					if ( %op_success ) {
						&list_status($detail_level, %op_success);
					}
				}
			}
			$webpage->print("</table>\n");
			if ( $detail_level eq "disabled" ) {
				if ( ( scalar(@disabled_groups) > 0 || 
				scalar(@disabled_hosts) > 0 || scalar(@disabled_svcs) > 0 )) {
					print $webpage->end_form();
				}
			}else{
				# Print the legend below the table
				&moncgi_print_service_table_legend ();
			}
		}
	}
	# NPG end
}

# This subroutine tests whether a given group is allowed to be
# shown to the user.
# Inputs:      Name of group to check
# Outputs:  1  Group is allowed to be shown
#           0  Group is not allowed to be shown
sub can_show_group {
	my ($group) = (@_);
	my $watch;
	# Do not print out the status for this group unless it is on the 
	# "allowed" list
	if (@show_watch) {	# user defined one or more watch keywords
		# Loop through each access control and look for a match
		foreach $watch (@show_watch) {
			if ( $group =~ m/^$watch$/ )  {	# we found a match
				return 1;
			}
		}
		return 0;
	} else {	# user didn't define any watch keywords, so show everything
		return 1;
	}
}

sub list_status {
	# NPG
	# This function preps for display the status of all hosts and
	# services. It is still kind of a mess, but it has to do a lot. It
	# could still be cleaned up though.
	# Splitting the processing from the output generation helped.
	# NPG End
	my ($detail_level, %op) = (@_);
	my (%group_list, $group, $service, $s, $g, $h);
	my %d = &mon_list("disabled","");
	my $servertime = &mon_servertime;
	# NPG
	# Special hashes of arrays for "OK, Non-Disabled Services"
	my (%ONDS, %ONDS_lastcheck, %ONDS_nextcheck, %ONDS_service) ;
	my ( $tcc, $tca );	# Test Column Vars
	my $lccol;	# Last Check Column contents
	my $nccol;	# Next Check Column contents
	my $gstat;	# Group Status
	my $sstat;	# Service Status
	my ( $gst, $sst ) = "";	# Group & Service Sup Txt
	my $AS;		# Alerts Sent
	# NPG End
	my %OPSTAT = %Mon::Client::OPSTAT;
	my ($service_acked_string, $ackcomment) ;
	my $failure_string;
	my $desc_string ;
	my %saw ;	# used for sorting
	my @disabled_hosts;
	foreach $group (sort keys %op) {	# begin group loop
		# Only show this group if we are allowed to see it
		next unless &can_show_group($group) ;
		# NPG
		$gstat = "";
		$gst = "";
		# NPG End
		if ($detail_level eq "full") {
			# get a list of members of the group if we haven't already
			# NPG
			$group_list{$group} = [ &mon_list( "group", $group) ] unless $group_list{$group};
			# Why was this in the service loop?
			# Strip the * from disabled hosts
			foreach ( @{$group_list{$group}}) {
				if ($_ =~ /^\*/) {
					$_ =~ s/^\*//;
				}
			}
			$gst = "(" . join(", ",@{$group_list{$group}}) . ")<br/>\n";
			# NPG End
		}
		# NPG
		# Why was this in the service loop!
		foreach $g (keys %{$d{"hosts"}}) {
			foreach $h (keys %{$d{"hosts"}{$group}}) {
				push(@disabled_hosts , $h);
			}
		}
		# uniq and sort the returned array of disabled hosts
		@saw{@disabled_hosts} = ();
		@disabled_hosts = sort keys %saw;
		# Not as concise as Oringinal Code, but easier to follow.
		if ( scalar(@disabled_hosts) > 0 ) {
			$gst .= "(". join(" " , @disabled_hosts) . " DISABLED)\n";
			$gstat = -1;
		}
		# Watch Disabled Check
		if (${d{"watches"}{$group}}) {
			$gstat = 0;
		}
		if ( $gstat eq "" ) {
			$gstat = 1;
		}
		# NPG End
		foreach $service (sort keys %{$op{$group}}) {	# begin service loop
			$s = \%{$op{$group}->{$service}};
			undef %saw;
			undef @disabled_hosts;
			# NPG
			# Unified output prep code ( Col 2 )
			if ($s->{"opstatus"} == $OPSTAT{"fail"}) {
				# If no alerts issued, then we call this
				# service "failing" instead of failed, on the
				# assumption that if it hasn't generated an
				# alert yet, it isn't "really" important,
				# although you'd still like to know about it.
				$AS = $s->{"alerts_sent"};
				if ( $AS == 0 ) {
					$AS = "NO";
					$sstat = -1;
				} else {
					$sstat = -2;
					if ( $detail_level ne "full" ) {
						$AS = "";
					}
				}
				$failure_string = "FAILED";
				if ( $detail_level eq "full" ) {
					$failure_string .= " " . $AS .
					" Alerts Sent";
				}else{
					$failure_string .= ",${AS}ALERTS" ;
				}
				# Assemble the ACK message, if any.
				# Escape the HTML to avoid any potential
				# nastiness if the user requested it,
				# otherwise, just pass it on through as is.
				if ( $op{$group}{$service}{'ack'} != 0 ) {
					$sstat = -1;
					if ($untaint_ack_msgs) {	# We untaint
						$ackcomment = $op{$group}{$service}{'ackcomment'} eq "" ? "(no ack msg)" : HTML::Entities::encode_entities($op{$group}{$service}{'ackcomment'}) ;
					} else {		# We don't untaint
						$ackcomment = $op{$group}{$service}{'ackcomment'} eq "" ? "(no ack msg)" : $op{$group}{$service}{'ackcomment'} ;
					}
					$service_acked_string = " (ACKED: $ackcomment)" ;
				}else{
					$service_acked_string = "";
				}
			}else{
				$failure_string = "";
				$AS = "";
				if ($s->{"opstatus"} == $OPSTAT{"untested"})  {
					$sstat = -3;
				} elsif ($s->{"opstatus"} == $OPSTAT{"ok"}) {
					$sstat = 1;
				}
			}
			if (${d{"services"}{$group}{$service}}) {
				if ($sstat == -3) {
					$sstat = -4;
				}elsif ( $sstat == -1 || $sstat == -2 ) {
					$sstat = -5;
				}else{
					$sstat = 0;
				}
			}
			# Unified output prep code ( Col 3 & 4 )
			if ($s->{"last_check"}) {
				# Service IS NOT a trap
				$lccol = &pp_sec_brief($s->{"last_check"} - $servertime);
				if ($s->{"opstatus"} == $OPSTAT{"fail"}) {
					# If service is failing, add last_OK
					# time. We need to check if the var is
					# defined, since traps can throw us off.
					# Split Last OK output into two lines.
					$lccol .= "<br/>Last OK:<br/>";
					if ( (!defined($s->{"last_success"}))
					|| ($s->{"last_success"} == 0) ) {
						# Has no last_success time
						$lccol .= "Never";
					} else {
						# Has last_success time
						$lccol .= &pp_sec_brief($s->{"last_success"} - $servertime);
					}
				}
			} else {
				# Service IS a trap
				$lccol = "--";
			}
			if ( ( $s->{"timer"} == 0 ) || (!($s->{"last_check"})) ) {
				# Service IS a trap or timer is 0
				$nccol = "--";
			} else {
				# Service IS NOT a trap
				$nccol = &pp_sec_brief($s->{"timer"});
			}
			if ($detail_level ne "full") {
				# Brief Mode On
				$desc_string = "";
				if ( ($s->{"opstatus"} == $OPSTAT{"ok"}) 
				&& ( $sstat != 0 && $sstat != -4 ) ){
					# ONDS Check passed.
					# Load ONDS arrays
					push (@{ $ONDS{$group} }, "<a href=\"$cmd_url=svc_details&amp;args=$group,$service\">$service</a>");
					# Make sure we have a service variable.
					push (@{ $ONDS_service{$group} }, $service);
					push (@{ $ONDS_lastcheck{$group} }, $lccol);
					push (@{ $ONDS_nextcheck{$group} }, $nccol);
					# Next Item.
					next;
				}
			}else{
				# Full Mode On
				# Only def this when we need it.
				my %desc = &mon_list("descriptions","");
				$desc_string = ($desc{$group}{$service} ne "") ? " &quot;$desc{$group}{$service}&quot;" : " &lt;no description specified&gt;" ;
			}
			# Unified output prep code ( Col 2 )
			# NPG End
			if ($s->{"opstatus"} == $OPSTAT{"untested"})  {
				# NPG
				$sst = $desc_string;
				# NPG End
			} elsif ($s->{"opstatus"} == $OPSTAT{"fail"}) {
				# NPG
				$sst = "${desc_string} : \n<font size=\"+1\">$s->{last_summary}</font>\n<br/>($failure_string) ${service_acked_string}";
				# NPG End
			} elsif ($s->{"opstatus"} == $OPSTAT{"ok"}) {
				# NPG
				$sst = $desc_string;
				# NPG End
			} else {
				my $txt = "";
				for (keys %OPSTAT) {
					$txt = $_ if ($s->{"opstatus"} == $OPSTAT{$_});
				}
				# NPG
				$sst = "(details: $txt)";
				# NPG End
			}
			# NPG
			# Unified output prep code ( Col 5 )
			$tcc = "service";
			$tca = "$group,$service";
			&print_status ($detail_level, $gstat, $sstat, $group, $service, $lccol, $nccol, $tcc, $tca, $gst, $sst );
			# NPG End
		} # end $service loop
		# NPG
		# NEW: prep a "compressed" version of OK, Non-Disabled
		# Services (ONDS), the assumption being that ONDS's are not
		# all that interesting, so don't use up a lot of screen space
		# displaying them. The whole thing is contingent upon us being
		# in "brief" mode
		# NPG End
		if ($detail_level ne "full") {
			# NPG
			# Prep the ONDS's for this group if any exist.
			if ( $ONDS{$group} ) {
				# Unified output prep code
				$sstat = 1;
				my $svc = join(", ", @{$ONDS{$group}});
				$lccol = join(", ", @{$ONDS_lastcheck{$group}});
				$nccol = join(", ", @{$ONDS_nextcheck{$group}});
				if ( scalar(@{ $ONDS_service{$group} }) > 1 ) {
					# For grouped services.
					$tcc = "all";
					$tca = $group;
				}else{
					# For single services.
					$service = @{$ONDS_service{$group}}[0];
					$tcc = "service";
					$tca = "$group,$service";
				}
				&print_status ($detail_level, $gstat, $sstat, $group, $svc, $lccol, $nccol, $tcc, $tca, $gst, $sst );
			# NPG End
			}
		}	# end of ONDS printing
	}		# end of $group loop
}

# NPG
sub print_status {
	# This is the new function that 90% of the time you will be viewing.
	# Split the output from the processing, that way things are cleaner.
	my ($detail_level, $gstat, $sstat, $group, $service, $lc, $nc, $tcc, $tca, $gst, $sst ) = (@_);
	my $bg_fail = $redlight_color;
	my $bg_fail_noalerts = $yellowlight_color;
	my $bg_ok = $greenlight_color;
	my $td_bg_color;
	my $face = $sans_serif_font_face;
	my $tdh = "<td align=\"left\" bgcolor=\"";
	my $tdt = "</font></td>\n";	# Table Data tail
	my ($gd, $sd) = "";		# Group & Service Status
	my $tcs;
	# Unified output code ( col 1 )
	if ( $gstat < 1 ) {
		$td_bg_color = $disabled_color;
		if ( $gstat == 0 ) {
			$gd = "(DISABLED)";
		}
	}else{
		$td_bg_color = $BGCOLOR;
	}
	$webpage->print("<tr>$tdh$td_bg_color\">\n");
	$webpage->print("<font face=\"$face\" size=\"+1\">\n");
	$webpage->print("$gd<a href=\"$cmd_url=query_group&amp;args=$group\">");
	$webpage->print("$group</a></font>");
	if ($gst ne ""){
		$webpage->print("<br/>\n$gst");
	}
	$webpage->print("</td>\n");
	# Unified output code ( col 2 )
	if ( $sstat < 1 ) {
		$td_bg_color = $disabled_color;
		if ( $sstat == 0 || $sstat == -4 || $sstat == -5) {
			$sd = "(DISABLED)";
		} elsif ( $sstat == -1 ) {
			$td_bg_color = $bg_fail_noalerts;
		} elsif ( $sstat == -2 ) {
			$td_bg_color = $bg_fail;
		} elsif ( $sstat == -3 ) {
			$td_bg_color = $unchecked_color;
		}
	} elsif ( $sstat == 1 ) {
		$td_bg_color = $bg_ok;
	} else {
		$td_bg_color = $BGCOLOR;
	}
	$webpage->print("$tdh$td_bg_color\">\n");
	if ($detail_level eq "full" || $sstat < 1 ) {
		$webpage->print("$sd<a href=\"$cmd_url=svc_details&amp;args=$group,$service\">");
	}
	if ($sstat < 0 ) {
		if ($sstat == -3 || $sstat == -4) {
			# Unchecked or Unchecked Disabled
			$webpage->print("<b>${service}</b> -- UNCHECKED</a>$sst");
		}else{
			# Failed or Failed Disabled Disabled +/- Alerts
			$webpage->print("<font size=\"+1\"><b>${service}</b></font></a>$sst");
		}
	}elsif ($sstat == 0 ) { # OK Disabled
		$webpage->print("${service}</a>");
	} elsif ($sstat == 1 ) { # OK
		if ($detail_level eq "full"){
			$webpage->print("${service}</a>$sst");
		}else{
			$webpage->print("$service");
		}
	} else {
		$webpage->print("$sd$service $sst");
	}
	$webpage->print("</td>\n");
	# Unified output code ( col 3 & 4 )
	$td_bg_color = $BGCOLOR;
	my $fs;
	if ($detail_level eq "full" || $sstat != 1 ){
		$fs="+1";
	}else{
		$fs="-1";
	}
	$webpage->print("<td align=\"left\"><font size=\"$fs\">");
	$webpage->print($lc.$tdt);
	$webpage->print("<td align=\"left\"><font size=\"$fs\">");
	$webpage->print($nc.$tdt);
	# Unified output code ( col 5 )
	$webpage->print("<td align=\"left\">");
	print $webpage->start_form(-action=>"$url");
	if ( $tca eq $group && $tcc eq "all" ) {
		$tcc = "moncgi_test_all";
		$tcs = "Test All";
	}elsif ( $tcc eq "service" ) {
		$tcc = "mon_test_service";
		$tcs = "Test";
	}else{
		$tcc = "";
		$tcs = "Fail";
	}
	$webpage->param('command', "$tcc");
	$webpage->param('args', "$tca");
	print $webpage->submit(-name=>"$tcs", -class=>'tb');
	print $webpage->hidden(-name=>'command',);
	print $webpage->hidden(-name=>'args',);
	print $webpage->end_form();
	$webpage->print("</td>\n");
	$webpage->print("</tr>\n");
}
# NPG End

sub query_group {
	# Print out info about the hosts in a particular hostgroup
	my ($args) = (@_);
	my $group = &validate_name ($args);
	my (%hosts, $host, $retval, $c, $e, $s, $service, $status, %op, $bgcolor, $msg, $ena_checked, $dis_checked) ;
	my %OPSTAT = %Mon::Client::OPSTAT;
	# Color to shade the cells depending on whether a user is allowed to
	# execute the disable/enable commands.
	my $disable_command_bgcolor = $auth_commands{'disable'}{'auth'} == 0 ? "bgcolor=\"$disabled_color\"" : "" ;
	my $enable_command_bgcolor = $auth_commands{'enable'}{'auth'} == 0 ? "bgcolor=\"$disabled_color\"" : "" ;
	# NPG
	my %cmd_auth;
	# Better Input validation and safer output.
	my $Valid = &validate_group ( $group );
	if ( !defined $group || $Valid == 0 ) {
		$webpage->print("<center>Group does not exist!<br/></center>\n");
		return 0;
	}elsif ( $Valid == -1 ) {
		return 0;
	}
	# NPG End
	my %d = &mon_list("disabled","") ;
	@_ = &mon_list("group", $group);
	# turn the array into a hash, which is what we really want
	foreach (@_) {
		$hosts{$_} = "";
	}
	$webpage->print("<center>");
	# Only show the rest of this page if we are allowed to see 
	# information about this group.
	if ($show_watch_strict) {
		unless ( &can_show_group($group) ) {
			&moncgi_authfail;
			return 0;
		}
	}
	# NPG
	# make this link into a form.
	print $webpage->start_form(-action=>"$url");
	$webpage->param('command','query_group');
	$webpage->param('args', "$group");
	print $webpage->hidden(-name=>'command',);
	print $webpage->hidden(-name=>'args',);
	print $webpage->submit(-name=>'Reload this page now.');
	print $webpage->end_form();
	# NPG End
	$webpage->print("</center><br/>\n");
	# The table is divided into 3 sections: hostgroup, hosts, and services
	# NPG
	# Rebuild this whole page to make formalizing easier.
	# Table Width in CSS
	# Hostgroup changed to Host Group
	# Start the table
	$webpage->print("<table align=\"center\" class=\"group\" border=\"1\">");
	# Print the Group portion of the table
	$webpage->print("<tr><th>Host Group &quot;<em>$group</em>&quot;</th>");
	$webpage->print("<th>Downtime</th><th>Test</th><th>Status</th><th>Toggle</th></tr>\n");
	# NPG End
	$webpage->print("<tr>");
	# NPG
	$bgcolor = ( ${d{"watches"}{$group}} ) ? $disabled_color : $greenlight_color;
	# hostgroup changed to group.
	# Removed last reference to $group as it is at the start of the line.
	$webpage->print("<td>$group</td><td><a href=\"$cmd_url=list_dtlog&amp;args=$group\">Show Log</a></td>");
	$webpage->print("<td><a href=\"$cmd_url=moncgi_test_all&amp;args=$group\">Test all services</a></td>");
	$webpage->print("<td bgcolor=\"$bgcolor\"></td>");
	# Group Toggle Vars
	my $t_action;
	if (${d{"watches"}{$group}} ) {
		$t_action = "enable";
	}else{
		$t_action = "disable";
	}
	if ( $Toggle ) {
		# Offer group toggle.
		$webpage->print("<td>");
		print $webpage->start_form(-action=>"$url");
		$webpage->param('args', "watch,$group,$service");
		$webpage->param('rt', 'query_group');
		$webpage->param('rtargs', "$group");
		if ( $t_action eq "enable" ) {
			$webpage->param('command','mon_enable');
			print $webpage->submit(-name=>'ENABLE');
		} else{
			$webpage->param('command','mon_disable');
			print $webpage->submit(-name=>'DISABLE');
		}
		print $webpage->hidden(-name=>'command',);
		print $webpage->hidden(-name=>'args',);
		print $webpage->hidden(-name=>'rt',);
		print $webpage->hidden(-name=>'rtargs',);
		print $webpage->end_form();
		$webpage->print("</td>\n");
	}
	# NPG End
	$webpage->print("</tr>\n");
	# Print the hosts portion of the table
	$webpage->print("<tr>");
	# NPG
	# Removed reference to $group as it is in the previous two lines.
	$webpage->print("<th>Members of group</th><th colspan=\"4\"></th></tr>");
	# NPG End
	foreach $host (keys %hosts) {
		if ($host =~ /^\*/) {
			$host =~ s/^\*// ;		# strip the * or else mon dies
			$hosts{$host} = "disabled";
		} else {
			$hosts{$host} = "enabled";
		}
	}
	foreach $host (sort keys %hosts) {
		next if ($host =~ /^\*/);		# ignore disabled hosts
		$webpage->print("<tr>");
		if ($hosts{$host} eq "disabled") {	# check to see if the host is disabled
			# the host is currently disabled
			$host =~ s/^\*// ;	# strip the * or else mon dies (should already be gone but just in case, this is a bad thing to happen)
			# NPG
			$bgcolor = $disabled_color;
			$t_action = "enable";
			$webpage->print("<td colspan=\"3\">$host (DISABLED)</td>");
			# NPG End
		} else {	# host is currently enabled
			# NPG
			$bgcolor = $greenlight_color;
			$t_action = "disable";
			$webpage->print("<td colspan=\"3\">$host</td>");
			# NPG End
		}
		# NPG
		$webpage->print("<td bgcolor=\"$bgcolor\"></td>");
		if ( $Toggle ) {
			# Offer host toggle.
			$webpage->print("<td>");
			print $webpage->start_form(-action=>"$url");
			$webpage->param('args', "host,$host");
			$webpage->param('rt', 'query_group');
			$webpage->param('rtargs', "$group");
			if ( $t_action eq "enable" ) {
				$webpage->param('command','mon_enable');
				print $webpage->submit(-name=>'ENABLE');
			} else{
				$webpage->param('command','mon_disable');
				print $webpage->submit(-name=>'DISABLE');
			}
			print $webpage->hidden(-name=>'command',);
			print $webpage->hidden(-name=>'args',);
			print $webpage->hidden(-name=>'rt',);
			print $webpage->hidden(-name=>'rtargs',);
			print $webpage->end_form();
			$webpage->print("</td>\n");
		}
		# NPG End
		$webpage->print( "</tr>\n") ;
	}
	# Print the services portion of the table
	$webpage->print("<tr>");
	# NPG
	# Changed "Services monitored on hostgroup" to "Services in Group"
	# hostgroup completely eliminated in test link.
	# Removed Enabled & Disabled headers as they are in service sub table.
	$webpage->print("<th>Services in group \"<em>$group</em>\"</th><th colspan=\"4\"></th>");
	# NPG End
	$webpage->print("</tr>\n");
	# NPG
	# Service Sub Table headers.
	$webpage->print("<tr>");
	$webpage->print("<th>Service</th><th>Downtime</th>");
	$webpage->print("<th>Test</th><th>Status</th><th>Toggle</th>");
	$webpage->print("</tr>\n");
	%op = &mon_list ("opstatus", "$group");
	# NPG End
	foreach $service (sort keys %{$op{$group}}) {		# begin service loop
		$s = \%{$op{$group}->{$service}};
		undef $bgcolor;
		if (${d{"services"}{$group}{$service}}) {	# service is disabled
			$bgcolor = "$disabled_color";
			# NPG
			$t_action = "enable";
			$msg="(status: DISABLED)";
		}else{
			$t_action = "disable";
			if ($s->{"opstatus"} == $OPSTAT{"ok"}) {
				# OK
				$bgcolor = "$greenlight_color";
				$msg="(status: OK)";
			# NPG End
			} elsif ($s->{"opstatus"} == $OPSTAT{"fail"}) {
				# NPG
				# Removed unless ($bgcolor) around the if 
				# clause & added it where needed to make stuff 
				# look right.
				# Now the fail messages should match between 
				# the main page & the group view.
				# Just need to get the other stuff synced.
				if ( $s->{"alerts_sent"} == 0 ) {
					$bgcolor = "$yellowlight_color";
					$msg=": $s->{'last_summary'}<br/>(status: FAILED,NOALERTS)";
				} else {
					$bgcolor = "$redlight_color";
					$msg=": $s->{'last_summary'}<br/>(status: FAILED,ALERTS)";
				}
				if ($s->{"ack"} != 0 ) {
					$bgcolor = "$yellowlight_color";
					$msg .= "(ACKED: ";
					if ( $s->{'ackcomment'} ne "" ) {
						$msg .= "$s->{'ackcomment'})";
					} else {
						$msg .= "no ack msg)";
					}
				}
				# NPG End
			} elsif ($s->{"opstatus"} == $OPSTAT{"untested"}) {
				$bgcolor = "$unchecked_color";
				$msg="(status: UNTESTED)";
			}
		# NPG
		}
		# Service Sub Table
		# Old Code put everything into a single cell.
		# Breaking it out to a table actually makes the display cleaner.
		# Also elininating excessive references to $group and Service 
		# helped.
		# also made the <input> tags shtml compliant.
		$webpage->print("<tr>");
		$webpage->print("<td><a href=\"$cmd_url=svc_details&amp;args=$group,$service\">$service</a></td>") ;
		$webpage->print("<td><a href=\"$cmd_url=list_dtlog&amp;args=$group,$service\">Show Log</a></td>") ;
		$webpage->print("<td><a href=\"$cmd_url=mon_test_service&amp;args=$group,$service\">Test</a></td>");
		$webpage->print("<td bgcolor=\"$bgcolor\">$msg</td>") ;
		if ( $Toggle ) {
			# Offer service toggle.
			$webpage->print("<td>");
			print $webpage->start_form(-action=>"$url");
			$webpage->param('args', "service,$group,$service");
			$webpage->param('rt', 'query_group');
			$webpage->param('rtargs', "$group");
			if ( $t_action eq "enable" ) {
				$webpage->param('command','mon_enable');
				print $webpage->submit(-name=>'ENABLE');
			} else{
				$webpage->param('command','mon_disable');
				print $webpage->submit(-name=>'DISABLE');
			}
			print $webpage->hidden(-name=>'command',);
			print $webpage->hidden(-name=>'args',);
			print $webpage->hidden(-name=>'rt',);
			print $webpage->hidden(-name=>'rtargs',);
			print $webpage->end_form();
			$webpage->print("</td>\n");
		}
		# NPG End
		$webpage->print("</tr>\n");
	}
	# NPG
	# Wrap up Service Sub Table.
	# NPG End
	$webpage->print("</table>");
}

sub end_page {
	# End the document with a footer and contact info
	# NPG
	# Fix this for XHTML Compliance
	$webpage->print("<table border=\"0\" summary=\"Page Footer\"><tr>");
	# NPG End
	if ($monadmin ne "") {
		# NPG
		# Fix this for XHTML Compliance
		$webpage->print("<td align=\"left\">For questions about this server,<br/>contact <a href=\"mailto:$monadmin\">$monadmin</a></td>");
		# NPG End
	}
	# NPG
	# At least print the moncgi version.
	$webpage->print("<td align=\"right\"><i>mon.cgi v $VERSION</i></td></tr></table>");
	# NPG End
	print $webpage->end_html;
}

sub list_alerthist {
	# This function lists the alert history formatted in a table
	# NPG
	if ( !$ListLock ) {
	# NPG End
		my @l = &mon_list ("alert history", "" ) ;
		my ($line, $localtime);
		# NPG
		# Orig. code moved from mon_list_alerthist
		# Why? This is the presentation sub, that was the data gathering sub.
		# This is presentation code.
		print $webpage->hr;
		print $webpage->h2("Alert History:");
		# Fix this for XHTML Compliance
		$webpage->print("<table border=\"1\" class=\"alerthist\" align=\"center\">\n");
		# NPG End
		$webpage->print("<tr><th>Entry</th><th>Group</th><th>Service</th>");
		$webpage->print("<th>Type</th><th>Time</th><th>Alert</th><th>Args</th>");
		$webpage->print("<th>Summary</th></tr>\n");
		# NPG
		# Entry #
		my $Entry = 0;
		# NPG End
		foreach $line (reverse sort {$a->{"time"} <=> $b->{"time"}} (@l)) {
			# Only show the alert if we are allowed to see information
			# about this group.
			if ($show_watch_strict) {
				next unless &can_show_group($line->{group});
			}
			# NPG
			# Add Entry #'s to table
			$Entry ++;
			$webpage->print("<tr><td>$Entry</td><td><a href=\"$cmd_url=query_group&amp;");
			# NPG End
			$webpage->print("args=$line->{group}\">$line->{group}</a></td>");
			$webpage->print("<td>$line->{service}</td><td>$line->{type}</td>");
			# NPG
			# Standardize TimeStamp
			$localtime = &TimeStamp("TD",$line->{"time"});
			# NPG End
			$webpage->print("<td>$localtime</td>\n");
			$line->{"alert"} =~ s{^.*\/([^/]*)$}{$1};
			$webpage->print("<td>$line->{alert}</td>\n");
			my $args = "-";
			if ($line->{"args"} !~ /^\s*$/) {
				$args = $line->{"args"};
			}
			$webpage->print("<td>$args</td><td>$line->{summary}</td></tr>\n");
		}
		$webpage->print("</table>\n");
	# NPG
	}
	# NPG End
}

sub svc_details {
	# Lists details about a particular alert's status, regardless of
	# whether the alert is successful or not.
	# As of 1.40, this function has been expanded considerably, and
	# has also been given the benefit of additional verbiage from the
	# global variables %alert_vars, @time_based_alert_vars, and 
	# @pp_sec_alert_vars
	my ($arg) = (@_);
	my ($group, $service) = split (/\,/, $arg);
	# NPG
	# Validate our input.
	my $Valid = &validate_gs ( $group,$service );
	if ( $Valid == 0 ) {
		&setup_page("Service Details", 400);
		$webpage->print("<center>Group or Service does not exist!<br/></center>\n");
		return 0;
	}elsif ( $Valid == -1 ) {
		&setup_page("Service Details", 403);
		return 0;
	}
	# NPG End
	# Only show the rest of this page if we are allowed to see
	# information about this group.
	if ($show_watch_strict) {
		unless ( &can_show_group($group) ) {
			# NPG
			&setup_page("Service Details", 403);
			# NPG End
			&moncgi_authfail;
			return 0;
		}
	}
	# NPG
	&setup_page("Service Details", 200);
	# NPG End
	my $status;
	my $retval;
	my (@pids, $server, $acknowledge_string, $name_string, $ackcomment_default);
	my (%op, $s, $g, $var);
	my $font_color;
	my %d = &mon_list("disabled","");
	my %desc = &mon_list("descriptions","");
	my $servertime = &mon_servertime;
	my @group_members = &mon_list("group", $group) ;
	my $time_now = $servertime;
	# NPG
	# Custom Variables
	my ( $svc_state, $group_state, $ack_desc, %cmd_auth );
	my @cmds = qw ( enable disable test ack );
	# Get cmd auths
	foreach my $tmp ( @cmds ) {
		$cmd_auth{$tmp} = $auth_commands{$tmp}{'auth'} == 0 ? "bgcolor=\"$disabled_color\"" : "" ;
	}
	# NPG End
	my $disabled_hosts_string ;
	# Determine whether the service is failing or not
	# We'll be optimistic and assume it's NOT failing :)
	# Actually, unchecked services also show up as "ok".
	%op = &mon_list("successes", "$group,$service");
	if ($op{$group}{$service}) {
		$status = "ok";
	} else {
		$status = "fail";
		%op = &mon_list("failures", "$group,$service");
	}
	print $webpage->hr;
	$webpage->print("<center>");
	# NPG
	# Unify output gen Code.
	# Control the colored text with span not font.
	# Group status is displayed in all messages using color.
	# Now the first line of service details will give you a quick visual 
	# of the Group and Service state.
	if ( ${d{"watches"}{$group}} ) {
		$group_state = "disabled";
	}else{
		$group_state = "ok";
	}
	$webpage->print("<font size=\"+2\">Detail for ");
	if (${d{"services"}{$group}{$service}}) {	# service is disabled
		$svc_state = "disabled";
	} elsif ($op{$group}{$service}{'opstatus'} == 1) {	# OK
		$svc_state = "ok";
	} elsif ($op{$group}{$service}{'opstatus'} == 7) {	# service is Untested
		$svc_state = "unchecked";
	} else {# service is Failed (or some other opstatus code I don't know about)
		if ( $op{$group}{$service}{'ack'} != 0 ) {
			$svc_state = "acked";
		}else{
			if ( $op{$group}{$service}{'alerts_sent'} == 0 ) {
				$svc_state = "failing";
			}else{
				$svc_state = "failed";
			}
		}
	}
	$webpage->print(" $group_state group 
	<span class=\"$group_state\">$group</span> and $svc_state service 
	<span class=\"$svc_state\"><i>$service</i>.</span></font>\n");
	if ($status eq "fail") {
		# If a service fails, print the detail and summary information
		# at the top. Yes, it's buried down in the middle of the page, but
		# it's important enough to take up screen real estate with and 
		# bring it to the top.
		$webpage->print("<table class=\"svcdet\" border=\"0\" align=\"center\" cellpadding=\"2\" cellspacing=\"2\">\n");
		# Now print the detail and summary information for the failed service
		$op{$group}->{$service}->{'last_summary'} = "&lt;not specified&gt;" if $op{$group}->{$service}->{'last_summary'} eq "" ;
		$op{$group}->{$service}->{'last_detail'} = "&lt;not specified&gt;" if $op{$group}->{$service}->{'last_detail'} eq "" ;
		$op{$group}->{$service}->{'last_detail'} =~ s/\n/<br\/>/g;
		$webpage->print("<tr><td width=\"25%\"><font size=\"+1\" ><span class=\"$svc_state\">Failure summary</span></font>:</td><td>$op{$group}->{$service}->{'last_summary'}</td></tr>\n");
		$webpage->print("<tr><td width=\"25%\"><font size=\"+1\" ><span class=\"$svc_state\">Failure detail</span></font>:</td><td>$op{$group}->{$service}->{'last_detail'}</td></tr>\n");
		$webpage->print("</table>");
	}
	# NPG End
	# Issue warning if any hosts in the group are currently disabled.
	foreach (@group_members) {
		if ($_ =~ s/^\*//) {
			$disabled_hosts_string = " $_" . $disabled_hosts_string ;
		}
	}
	# NPG
	# Make this a bit more concise
	if ( $disabled_hosts_string ne "") {
		$webpage->print ("<br/><font size=\"+1\">The following host(s) in this group are disabled: <span class=\"disabled\">$disabled_hosts_string</span></font>");
	}
	# NPG end
	# Check to see if a test for this service is currently running
	# and report back if it is (because the status might not have updated
	# yet...)
	@pids = &mon_list("pids","");
	# NPG
	if (!@pids) {
		$webpage->print("<font size=\"+1\">Unable to to determine whether this service is currently being tested (list_pids failed)!</font>\n");
	# NPG End
	} else {
		shift @pids;	# discard server PID, we don't need it
		for (@pids) {
			if ( ($_->{"watch"} eq $group) && ($_->{"service"} eq $service) ) {
				# we have a match, a monitor is currently 
				# running for this group and this service.
				# NPG
				$webpage->print("<br/><font size=\"+1\">A 
				monitor for this service is currently running 
				as PID $_->{'pid'}.<br/>Results/opstatus codes 
				might change when this test 
				finishes.</font>\n");
				# NPG End
			}
		}
	}
	$webpage->print("<br/>");
	# NPG
	# make this link into a form.
	print $webpage->start_form(-action=>"$url");
	$webpage->param('command','svc_details');
	$webpage->param('args', "$group,$service");
	print $webpage->hidden(-name=>'command',);
	print $webpage->hidden(-name=>'args',);
	print $webpage->submit(-name=>'Reload this page now.');
	print $webpage->end_form();
	# NPG End
	$webpage->print("</center><br/>");
	# NPG
	# Fix this for XHTML Compliance
	$webpage->print("<table class=\"svcdet\" align=\"center\" border=\"1\"><tr>");
	# Made this Service Action bar more concise by removing  references to
	# $service & $group in hyperlinks
	# Turn the links into forms.
	$webpage->print("<td>Service Actions</td>\n");
	if ( $cmd_auth{'test'} eq "" ){
		$webpage->print("<td>");
		print $webpage->start_form(-action=>"$url");
		$webpage->param('command','mon_test_service');
		$webpage->param('args', "$group,$service");
		print $webpage->hidden(-name=>'command',);
		print $webpage->hidden(-name=>'args',);
		print $webpage->submit(-name=>'Test', -class=>'tb');
		print $webpage->end_form();
		$webpage->print("</td>\n");
	}
	# Service Toggle Vars
	my $t_action;
	my $t_auth;
	if (${d{"services"}{$group}{$service}} ) {
		$t_action = "enable";
	}else{
		$t_action = "disable";
	}
	if ( $cmd_auth{$t_action} eq "" ) {
		$t_auth = 1;
	}else{
		$t_auth = 0;
	}
	if ( $t_auth ) {
		# Offer service toggle.
		$webpage->print("<td>");
		print $webpage->start_form(-action=>"$url");
		$webpage->param('args', "service,$group,$service");
		$webpage->param('rt', 'svc_details');
		$webpage->param('rtargs', "$group,$service");
		if ( $t_action eq "enable" ) {
			$webpage->param('command','mon_enable');
			print $webpage->submit(-name=>'ENABLE');
		} else{
			$webpage->param('command','mon_disable');
			print $webpage->submit(-name=>'DISABLE');
		}
		print $webpage->hidden(-name=>'command',);
		print $webpage->hidden(-name=>'args',);
		print $webpage->hidden(-name=>'rt',);
		print $webpage->hidden(-name=>'rtargs',);
		print $webpage->end_form();
		$webpage->print("</td>\n");
	}
	$webpage->print("<td>");
	print $webpage->start_form(-action=>"$url");
	$webpage->param('command','list_dtlog');
	$webpage->param('args', "$group,$service");
	$webpage->param('rt', 'svc_details');
	$webpage->param('rtargs', "$group,$service");
	print $webpage->hidden(-name=>'command',);
	print $webpage->hidden(-name=>'args',);
	print $webpage->hidden(-name=>'rt',);
	print $webpage->hidden(-name=>'rtargs',);
	print $webpage->submit(-name=>'Show downtime log');
	print $webpage->end_form();
	$webpage->print("</td>");
	# NPG End
	$webpage->print("</tr>\n");
	$webpage->print("</table>\n");
	# If the service is in a failure state, offer to ack it.
	# NPG
	if ( $status eq "fail" && $cmd_auth{'ack'} eq "" ) {
		$webpage->print("<table class=\"svcdet\" align=\"center\" border=\"0\">");
	# NPG End
		$webpage->print("<tr>");
		# NPG
		$webpage->print("<td colspan=\"1\" width=\"50%\">");
		# Unify output gen Code.
		if ($op{$group}{$service}{'ack'} != 0) {
			# Service has already been acked, offer to re-ack
			$acknowledge_string = "Re-a";
			$ack_desc = "Changes the acknowledgement message.";
			$ackcomment_default = "$op{$group}{$service}{'ackcomment'}";
		} else {
			# Service has not yet been acked, offer to ack
			$acknowledge_string = "A";
			$ack_desc = "Disables all subsequent alerts for this failure period.";
			$ackcomment_default = "${name_string}";
		}
		$webpage->print("<font size=\"+1\"><b>");
		$webpage->print("$acknowledge_string");
		$webpage->print("cknowledge this failure:</b></font><br/>");
		$webpage->print("$ack_desc<br/></td>");
		# Fix this for XHTML Compliance
		$webpage->print("<td valign=\"top\">");
		# NPG End
		# It is crucial that we reset the param value, otherwise the
		# value of the default will be ignored.
		# NPG
		print $webpage->start_form(-action=>"$url");
		# NPG End
		$webpage->param('ackcomment', $ackcomment_default);
		print $webpage->textfield(-name=>'ackcomment',
			-value=>"$ackcomment_default", -size=>40, );
		$webpage->print("&nbsp;&nbsp;");
		# We also have to reset this param value
		$webpage->param('command','mon_ack');
		print $webpage->hidden(-name=>'command',);
		$webpage->param('args',"$group,$service");
		# We also have to reset this param value
		print $webpage->hidden(-name=>'args',);
		# NPG
		if ($op{$group}{$service}{'ack'} != 0) {
			print $webpage->submit(-name=>'Re-ack');
		}else{
			print $webpage->submit(-name=>'Ack');
		}
		# NPG End
		print $webpage->end_form();
		$webpage->print("</td></tr></table>\n");
	}
	# NPG
	# Fix these for XHTML Compliance
	$webpage->print("<table class=\"svcdet\" border=\"1\" align=\"center\" cellpadding=\"2\" cellspacing=\"2\">\n");
	$webpage->print("<tr><td align=\"left\" colspan=\"4\"><font size=\"+2\">Monitor Timers</font></td></tr>\n");
	foreach $var ( keys %{$op{$group}{$service}} ) {
	# NPG End
		# Special cases where we have a time formatted in secs
		# since 1970, we'll make it look purty
		if (grep /^${var}$/, @time_based_alert_vars) {
			# We need to check if the var is defined, since traps
			# can throw us off.
			if ( (!defined($op{$group}->{$service}->{$var})) || ($op{$group}->{$service}->{$var} == 0) ) {
				# The event has never occurred
				$op{$group}->{$service}->{$var} = "Never" ;
			} else {
				# Pretty-print the time
				$_ = $op{$group}->{$service}->{$var};
				$op{$group}->{$service}->{$var} = &TimeStamp ("ST",$op{$group}->{$service}->{$var});
				# Also calculate the delta and pretty-print that
				$op{$group}->{$service}->{$var} .= "<br/>(" . &pp_sec($time_now - $_) . " ago)";
			}
		} elsif ($op{$group}->{$service}->{$var} eq "") {
			# special case where value of $var is empty 
			#(i.e. mon has never seen the service fail)
			$op{$group}->{$service}->{$var} = "-";
		} elsif (grep /^${var}$/, @pp_sec_alert_vars) {
			if ( (!defined($op{$group}->{$service}->{$var})) || ($op{$group}->{$service}->{$var} == 0) ) {
				# The event has never occurred
				$op{$group}->{$service}->{$var} = "Never" ;
			} else {
				# Special case where time is a duration and 
				# should be pretty printed.
				$op{$group}->{$service}->{$var} = &pp_sec($op{$group}->{$service}->{$var});
			}
		}
		$op{$group}->{$service}->{$var} =~ s/\n/<br\/>/g;
	# NPG
	}
	# Orderd display of infromation
	my @po = qw ( group service depend exclude_period monitor 
	monitor_running opstatus last_opstatus exitval last_summary 
	last_detail);
	my @pto = qw ( interval timer last_check last_success);
	my @fo = qw ( alertdepend alerts_sent ack ackcomment );
	my @fto = qw ( first_failure last_failure failure_duration last_alert);
	# Monitor Timers Display Start
	$webpage->print("<tr>");
	foreach $var ( @pto ) {
		if ( $op{$group}->{$service}->{$var} eq "" ) {
			next;
		}
		$webpage->print("<td><font size=\"+1\"><b>$alert_vars{$var}</b> ($var)</font><br/>");
		$webpage->print("<font size=\"+1\">$op{$group}->{$service}->{$var}</font></td>");
	}
	$webpage->print("</tr>");
	# Monitor Timers Display Done
	$webpage->print("<tr><td align=\"left\" colspan=\"4\"><font size=\"+2\">Monitor Information</font></td></tr>\n");
	$webpage->print("<tr><td align=\"left\" colspan=\"3\"><font size=\"+2\"><b>Variable Description</b> (name)</font></td>\n");
	$webpage->print("<td align=\"left\"><font size=\"+2\"><b>Value</b></font></td></tr>\n");
	my $desc_string = ($desc{$group}{$service} ne "") ? "&quot;$desc{$group}{$service}&quot;" : "&lt;no description specified&gt;" ;
	# Fix this for XHTML Compliance
	$webpage->print("<tr><td align=\"left\" colspan=\"3\"><font size=\"+1\"><b>Service Description</b></font></td><td>$desc_string</td></tr>\n");
	# Stack the failure detail vertically.
	# Add visual indicators for depstatus, last_opstatus, opstatus
	# & exitval.
	foreach $var ( @po ) {
		if ( $op{$group}->{$service}->{$var} eq "" ) {
			next;
		}
		# Left Col / Special
		$webpage->print("<tr><td colspan=");
		# Last Detail Header
		if ( $alert_vars{$var} eq $alert_vars{last_detail} ) {
			$webpage->print("\"4\"");
		}else{
			$webpage->print("\"3\"");
		}
		$webpage->print("><font size=\"+1\">");
		if ( $alert_vars{$var} ) {
			$webpage->print("<b>$alert_vars{$var}</b> ");
		}
		$webpage->print("($var)</font></td>");
		# End Left Col / Special
		if ( $alert_vars{$var} eq $alert_vars{last_detail}) {
			$webpage->print("</tr><tr><td colspan=\"4\"");
		}else{
			$webpage->print("<td");
		}
		if ($alert_vars{$var} eq $alert_vars{depstatus}) {
			if ( $op{$group}->{$service}->{$var} eq 1) {
				$webpage->print(" bgcolor=\"$greenlight_color\">");
			}elsif ( $op{$group}->{$service}->{$var} eq 0) {
				$webpage->print(" bgcolor=\"$redlight_color\">");
			}
		}elsif ($alert_vars{$var} eq $alert_vars{exitval}
		|| $alert_vars{$var} eq $alert_vars{monitor_running}) {
			if ( $op{$group}->{$service}->{$var} eq 0) {
				$webpage->print(" bgcolor=\"$greenlight_color\"");
			}else{
				if ( $alert_vars{$var} eq $alert_vars{monitor_running}) {
					$webpage->print(" bgcolor=\"$unchecked_color\"");
				}else{
					$webpage->print(" bgcolor=\"$redlight_color\"");
				}
			}
		}elsif	($alert_vars{$var} eq $alert_vars{last_opstatus}
		|| $alert_vars{$var} eq $alert_vars{opstatus}) {
			if ( $op{$group}->{$service}->{$var} eq 1) {
				$webpage->print(" bgcolor=\"$greenlight_color\"");
			}elsif ( $op{$group}->{$service}->{$var} eq 0) {
				$webpage->print(" bgcolor=\"$redlight_color\"");
			}elsif ( $op{$group}->{$service}->{$var} eq 7) {
				$webpage->print(" bgcolor=\"$unchecked_color\"");
			}
		}
		$webpage->print(">");
		if ( $alert_vars{$var} eq $alert_vars{last_detail}) {
			$webpage->print("<pre>$op{$group}->{$service}->{$var}</pre>");
		}else{
			$webpage->print("<font size=\"+1\">$op{$group}->{$service}->{$var}</font>");
		}
		$webpage->print("</td></tr>\n");
	}
	# Fail / Alert / Ack Info Display
	if ($status eq "fail") {
		$webpage->print("<tr><td align=\"left\" colspan=\"4\"><font size=\"+2\">Failure &amp; Alert Timers</font></td></tr>\n");
		$webpage->print("<tr>");
		foreach $var ( @fto ) {
			if ( $op{$group}->{$service}->{$var} eq "" ) {
				$webpage->print("<td></td>");
				next;
			}
			$webpage->print("<td><font size=\"+1\"><b>$alert_vars{$var}</b> ($var)</font><br/>");
			$webpage->print("<font size=\"+1\">$op{$group}->{$service}->{$var}</font></td>");
		}
		$webpage->print("</tr>");
		$webpage->print("<tr><td align=\"left\" colspan=\"4\"><font size=\"+2\">Alert &amp; Ack. Information</font></td></tr>\n");
		$webpage->print("<tr><td align=\"left\" colspan=\"3\"><font size=\"+2\"><b>Variable Description</b> (name)</font></td>\n");
		$webpage->print("<td align=\"left\"><font size=\"+2\"><b>Value</b></font></td></tr>\n");
		foreach $var ( @fo ) {
			if ( $op{$group}->{$service}->{$var} eq "" ) {
				next;
			}
			$webpage->print("<tr><td colspan=\"3\"><font size=\"+1\">");
			if ( $alert_vars{$var} ) {
				$webpage->print("<b>$alert_vars{$var}</b> ");
			}
			$webpage->print("($var)</font></td>");
			$webpage->print("<td>");
			$webpage->print("<font size=\"+1\">$op{$group}->{$service}->{$var}</font>");
			$webpage->print("</td></tr>\n");
		}
	}
	# Misc Info Display
	$webpage->print("<tr><td align=\"left\" colspan=\"4\"><font size=\"+2\">Misc. Information</font></td></tr>\n");
	foreach $var (sort {$b cmp $a } (keys %{$op{$group}{$service}}) ) {
		if ( (grep /^${var}$/, @po) || (grep /^${var}$/, @fo) 
		|| (grep /^${var}$/, @pto) || (grep /^${var}$/, @fto)) {
			next;
		}
		$webpage->print("<tr><td colspan=\"3\"><font size=\"+1\">");
		if ( $alert_vars{$var} ) {
			$webpage->print("<b>$alert_vars{$var}</b> ");
		}
		$webpage->print("($var)</font></td>");
		$webpage->print("<td");
		if ($alert_vars{$var} eq $alert_vars{depstatus}) {
			if ( $op{$group}->{$service}->{$var} eq 1) {
				$webpage->print(" bgcolor=\"$greenlight_color\">");
			}elsif ( $op{$group}->{$service}->{$var} eq 0) {
				$webpage->print(" bgcolor=\"$redlight_color\">");
			}
		}
		$webpage->print(">");
		$webpage->print("<font size=\"+1\">$op{$group}->{$service}->{$var}</font>");
		$webpage->print("</td></tr>\n");
		# NPG End
	}
	$webpage->print("</table>\n");
}

sub list_dtlog {
	# Accepts the following arguments, all of which are optional:
	# (group,service,sortby,first log entry to show, last log entry to show)
	# to sort by.
	#
	# Default sort key is failtime.
	# If {service,group} is null, shows detail about all failures
	# for the given {group,service}.
	#
	# Default first log entry to show is 1.
	# Default last log entry to show is 1+$dtlog_max_failures_per_page.
	#
	# No arguments means show all service failures for all groups, 
	# sorted by failtime.
	#
	# Someday it would be nice to take args like time ranges, etc., but
	# that capability should really be built into Mon itself since it
	# is such a useful feature and something which could leverage a lot
	# of mon's timeperiod work as well.
	# Original patch by Martha H Greenberg <marthag@mit.edu>
	my ($arg) = (@_);
	my ($group, $service,$sortby,$dtlog_begin,$dtlog_end);
	# NPG
	# Had to add the ne test when I tried to add input validation.
	# Appearantly an $arg of "" is still defined, but of course won't
	# validate since it doesn't exist in mon.
	# However in this case an $arg of "" is OK.
	if ( defined($arg) && $arg ne "" ) {
	# NPG End
		($group, $service,$sortby,$dtlog_begin,$dtlog_end) = split (/\,/, $arg) ;
		# NPG
		# Validate our input.
		my ( $OK, $OOps );
		$OK = 0;
		if ($group eq "" || $service eq "") {
			$OK = 1;
		}else{
			if ( !&validate_group ( $group ) ) {
				$OOps = "Group";
			}else{
				if ( !&validate_gs ( $group,$service ) ) {
					$OOps = "Group or Service";
				}else{
					$OK = 1;
				}
			}
		}
		if ( !$OK ) {
			$webpage->print("<center>$OOps does not exist!<br/></center>\n");
			return 0;
		}
		# NPG End
	} else {
		$group = "";
		$service = "";
		$sortby = "" ;
		$dtlog_begin = "";
		$dtlog_end = "";
	}
	$dtlog_begin = 1 unless ( ($dtlog_begin) && ($dtlog_begin > 0) );
	$dtlog_end = ($dtlog_begin + $dtlog_max_failures_per_page - 1) unless ( ($dtlog_end) && ($dtlog_end > 0) );
	$sortby = "failtime" unless ($sortby);
	my $face = $sans_serif_font_face;
	my $i;
	# This hash keeps track of the sortby keys and what 
	# their descriptions map to.
	my %sortby_key = ("group" => "Group",
	"service" => "Service",
	# NPG
	# More concise headings for dtlog
	"failtime" => "Event Start",
	"timeup" => "Event End",
	"downtime" => "Event Time",
	"interval" => "Test Interval",
	# NPG End
	"summary" => "Summary",
	);
	my ($line, $localtimeup, $localfailtime, $ppdowntime, $ppinterval);
	my ($first_failure_time, $total_failures, $mtbf, $mean_recovery_time, $median_recovery_time, $std_dev_recovery_time, $min_recovery_time, $max_recovery_time, @l) = &mon_list_dtlog($group, $service) ;
	my ($ppfft, $ppmtbf, $ppmean_recovery_time, $ppmedian_recovery_time, $ppmin_recovery_time, $ppmax_recovery_time, $ppstd_dev_recovery_time);
	# Estimated uptime calculation
	my $time_now = time;
	my $approx_uptime_pct = ( ( ($time_now - $first_failure_time + $mtbf ) > 0) && ( ($time_now - $first_failure_time + $mtbf - scalar(@l) * $mean_recovery_time > 0 ) ) ) ? sprintf("%.2f%", ( ( ($time_now - $first_failure_time + $mtbf) - (scalar(@l) * $mean_recovery_time) ) / ($time_now - $first_failure_time + $mtbf) ) * 100 ) : "&lt;not applicable&gt;";
	# NPG
	if ( $ListLock ) {
		return undef;
	}else{
		# Fix these for XHTML Compliance
		$webpage->print("<table border=\"1\" align=\"center\" class=\"dtsum\">\n");
		$webpage->print("<tr><th colspan=\"2\"><font size=\"+1\" face=\"$face\">\n");
		# Chenge hostGroup to Group
		$webpage->print("Downtime Summary For Group ");
		# NPG End
		if ($group eq "") {
			$webpage->print("<b>&lt;any&gt;</b>");
		} else {
			$webpage->print("<b>&quot;$group&quot;</b>");
		}
		$webpage->print(" and Service ");
		if ($service eq "") {
			$webpage->print("<b>&lt;any&gt;</b>");
		} else {
			$webpage->print("<b>&quot;$service&quot;</b>");
		}
		$webpage->print("</font></th></tr>\n");
		# NPG
		# Standardize TimeStamp
		$ppfft = &TimeStamp("ST",$first_failure_time);
		# NPG End
		$webpage->print("<tr><td>Log begins at:</td>\n<td>$ppfft</td>\n");
		$webpage->print("</tr>\n");
		$webpage->print("<tr><td>Total observed service failures:</td>\n<td>$total_failures</td>\n");
		$webpage->print("</tr>\n");
		$webpage->print("<tr><td>Mean time between service failures:</td>\n");
		$ppmtbf = &pp_sec($mtbf);
		$webpage->print("<td>$ppmtbf</td>\n");
		$webpage->print("</tr>\n");
		$webpage->print("<tr><td>Mean observed service failure time:</td>\n");
		$ppmean_recovery_time = &pp_sec($mean_recovery_time);
		$webpage->print("<td>$ppmean_recovery_time</td>\n");
		$webpage->print("</tr>\n");
		$webpage->print("<tr><td>Median observed service failure time:</td>\n");
		$ppmedian_recovery_time = &pp_sec($median_recovery_time);
		$webpage->print("<td>$ppmedian_recovery_time</td>\n");
		$webpage->print("</tr>\n");
		$webpage->print("<tr><td>Standard deviation of observed service failure times:</td>\n");
		$ppstd_dev_recovery_time = &pp_sec($std_dev_recovery_time);
		$webpage->print("<td>$ppstd_dev_recovery_time</td>\n");
		$webpage->print("</tr>\n");
		$webpage->print("<tr><td>Minimum observed service failure time:</td>\n");
		$ppmin_recovery_time = &pp_sec($min_recovery_time);
		$webpage->print("<td>$ppmin_recovery_time</td>\n");
		$webpage->print("</tr>\n");
		$webpage->print("<tr><td>Maximum observed service failure time:</td>\n");
		$ppmax_recovery_time = &pp_sec($max_recovery_time);
		$webpage->print("<td>$ppmax_recovery_time</td>\n");
		$webpage->print("</tr>\n");
		$webpage->print("<tr><td><i>Approximate</i> percentage of time in failure-free operation:</td>\n");
		$webpage->print("<td>$approx_uptime_pct</td>\n");
		$webpage->print("</tr></table>\n");
	# NPG
	}
	# NPG End
	return 0 if scalar(@l) == 0;	# stop if we returned no downtime events
	($dtlog_begin, $dtlog_end) = &moncgi_list_dtlog_navtable ($url, $group, $service, $sortby, $dtlog_begin, $dtlog_end, $total_failures, scalar(@l), %sortby_key) ;
	# Start printing the actual downtime log table.
	# NPG
	# Fix these for XHTML Compliance
	$webpage->print("<table border=\"1\" class=\"dt\" align=\"center\">\n");
	$webpage->print("<tr><th><font size=\"+1\">Entry</font></th>\n");
	$webpage->print("<th><font size=\"+1\"><a href=\"$cmd_url=list_dtlog&amp;args=$group,$service,group,$dtlog_begin,$dtlog_end\">$sortby_key{\"group\"}</a></font></th>\n");
	$webpage->print("<th><font size=\"+1\"><a href=\"$cmd_url=list_dtlog&amp;args=$group,$service,service,$dtlog_begin,$dtlog_end\">$sortby_key{\"service\"}</a></font></th>\n");
	$webpage->print("<th><font size=\"+1\"><a href=\"$cmd_url=list_dtlog&amp;args=$group,$service,failtime,$dtlog_begin,$dtlog_end\">$sortby_key{\"failtime\"}</a></font></th>\n");
	$webpage->print("<th><font size=\"+1\"><a href=\"$cmd_url=list_dtlog&amp;args=$group,$service,timeup,$dtlog_begin,$dtlog_end\">$sortby_key{\"timeup\"}</a></font></th>\n");
	$webpage->print("<th><font size=\"+1\"><a href=\"$cmd_url=list_dtlog&amp;args=$group,$service,downtime,$dtlog_begin,$dtlog_end\">$sortby_key{\"downtime\"}</a></font></th>\n");
	$webpage->print("<th><font size=\"+1\"><a href=\"$cmd_url=list_dtlog&amp;args=$group,$service,interval,$dtlog_begin,$dtlog_end\">$sortby_key{\"interval\"}</a></font></th>\n");
	$webpage->print("<th><font size=\"+1\"><a href=\"$cmd_url=list_dtlog&amp;args=$group,$service,summary,$dtlog_begin,$dtlog_end\">$sortby_key{\"summary\"}</a></font></th>\n");
	# NPG End
	$webpage->print("</tr>\n");
	# default sort order is "failtime", so no need to resort
	if ($sortby ne "failtime") {
		# do a forward-alphanumeric or reverse-numeric sort, 
		# depending on the sortby parameter
		if ( ($sortby eq "group") || ($sortby eq "service") || 
		($sortby eq "summary") ) {
			@l = (sort {$a->{"$sortby"} cmp $b->{"$sortby"}}(@l));
		} else {
			@l = (reverse sort {$a->{"$sortby"} <=> $b->{"$sortby"}}(@l));
		}
	}
	# Now print the rows of the downtime table
	for ( $i = $dtlog_begin ; $i <= $dtlog_end ; $i++ ) {
		$line = $l[$i-1];
		$webpage->print("<tr><td>$i</td>");
		$webpage->print("<td><a href=\"$cmd_url=list_dtlog&amp;args=$line->{\"group\"}\">");
		$webpage->print("<font face=\"$face\">$line->{\"group\"}</font></a></td>");
		$webpage->print("<td><a href=\"$cmd_url=list_dtlog&amp;args=$line->{\"group\"},$line->{\"service\"}\">");
		$webpage->print("<font face=\"$face\">$line->{\"service\"}</font></a></td>\n");
		# NPG
		# Standardize TimeStamp
		$localfailtime = &TimeStamp ("TD",$line->{"failtime"});
		# NPG End
		$webpage->print("<td>$localfailtime</td>\n");
		# NPG
		# Standardize TimeStamp
		$localtimeup = &TimeStamp ("TD",$line->{"timeup"});
		# NPG End
		$webpage->print("<td>$localtimeup</td>\n");
		$ppdowntime = &pp_sec_brief ($line->{"downtime"});
		$webpage->print("<td>$ppdowntime</td>\n");
		$ppinterval = &pp_sec_brief ($line->{"interval"});
		$webpage->print("<td>$ppinterval</td>\n");
		$webpage->print("<td>$line->{\"summary\"}</td>");
		$webpage->print("</tr>\n");
	}
	$webpage->print("</table>\n");
	&moncgi_list_dtlog_navtable ($url, $group, $service, $sortby, $dtlog_begin, $dtlog_end, $total_failures, scalar(@l), %sortby_key) ;
	undef @l;
}

sub list_pids {
	my $retval;
	# NPG
	my $Sched_Toggle_Cmd;	# Scheduler Toggle CMD
	print $webpage->hr;
	# Main Info Table
	$webpage->print("<table class=\"bar\"><tr><td width=\"33%\">");
	print $webpage->h2("Connection Info");
	$webpage->print("</td><td>");
	print $webpage->h2("Process Info");
	$webpage->print("</td><td>");
	print $webpage->h2("Control Panel");
	$webpage->print("</td></tr><tr><td>");
	# Server Info Table
	$webpage->print("<table class=\"pid\" border=\"1\"><tr>
	<th>Desc.</th><th>Value</th></tr>\n");
	$webpage->print("<tr><td>Connected:</td>");
	if( !$connect_failed ) {
		$webpage->print("<td bgcolor=\"$greenlight_color\">Yes");
	}else {
		$webpage->print("<td bgcolor=\"$redlight_color\">No");
	}
	$webpage->print("</td></tr>");
	$webpage->print("<tr><td>Host:</td>");
	$webpage->print("<td>" . $c->host() . "</td></tr>");
	$webpage->print("<tr><td>Port:</td>");
	$webpage->print("<td>" . $c->port() . "</td></tr>");
	$webpage->print("<tr><td>Protocol:</td>");
	$webpage->print("<td>" . $c->prot() . "</td></tr>");
	$webpage->print("<tr><td>Protocol ID:</td>");
	$webpage->print("<td>" . $c->protid() . "</td></tr>");
	$webpage->print("<tr><td>Protocol Ver:</td>");
	$webpage->print("<td>" . $c->version() . "</td></tr>");
	$webpage->print("</table></td><td>");
	$webpage->print("<table class=\"pid\" border=\"1\">\n");
	# Scheduler
	$webpage->print("<tr><td colspan=\"2\">Scheduler:</td>");
	if( $SchedState ) {
		$webpage->print("<td bgcolor=\"$greenlight_color\">Running");
		$Sched_Toggle_Cmd = "stop";
	}else {
		$webpage->print("<td bgcolor=\"$redlight_color\">Stopped");
		$Sched_Toggle_Cmd = "start";
	}
	$webpage->print("</td></tr>");
	# PID Table
	if ( !$ListLock ) {
		my @pids = &mon_list("pids","") ;
		# Turn this into an if / else. If we can't get the pids, there is no 
		# point in contiuing.  Also a bunch of cosmetic changes to the output.
		if ( !@pids ) {
			$webpage->print("<tr><td colspan=\"2\">\n");
			print $webpage->h2("Unable to list PID's on server!<br/>\n");
			$webpage->print("</td></tr>\n");
		}else{
			my $server = shift @pids;
			my $tmp = scalar(@pids);
			$webpage->print("<tr><th colspan=\"2\">Desc.</th><th>Value</th></tr>\n");
			$webpage->print ("<tr><td colspan=\"2\">Mon PID</td><td>$server</td>\n");
			$webpage->print("</tr>\n");
			if ( $tmp > 0 ) {
				$webpage->print("<tr><td colspan=\"3\">$tmp monitor(s) currently active.</td></tr>\n");
				$webpage->print("<tr><th>Group</th><th>Service</th><th>PID</th></tr>\n");
				for (@pids) {
					$webpage->print (join ("","<tr><td>",
					$_->{"watch"},"</td><td>",$_->{"service"},
					"</td><td>",$_->{"pid"},"</td></tr>\n",));
				}
			}
		}
		$webpage->print("</table></td><td>");
	}
	# Control Panel
	$webpage->print("<table class=\"pid\" border=\"1\"><tr>
	<th colspan=\"2\">Action</th></tr>\n");
	# Test Conf
	$webpage->print("<tr><td>Test Config</td><td>");
	print $webpage->start_form;
	$webpage->param('command',"mon_test_config");
	print $webpage->hidden(-name=>'command',);
	print $webpage->submit(-name=>'Go');
	print $webpage->end_form;
	$webpage->print("</td></tr>");
	#Scheduler
	if ( !$ListLock && ( $SchedCtl || $SchedLock )) {
		# Formalize Scheduler Control
		$webpage->print("<tr><td>" . ucfirst($Sched_Toggle_Cmd) . " Scheduler</td>");
		if ( $SchedCtl && $Sched_Toggle_Cmd ) {
			$webpage->print("<td>\n");
			print $webpage->start_form;
			$webpage->param('rt', 'list_pids');
			$webpage->param('rtargs', "");
			$webpage->param('command','mon_schedctl');
			$webpage->param('args',"$Sched_Toggle_Cmd");
			print $webpage->hidden(-name=>'command',);
			print $webpage->hidden(-name=>'args',);
			print $webpage->hidden(-name=>'rt',);
			print $webpage->hidden(-name=>'rtargs',);
			print $webpage->submit(-name=>"Go");
			print $webpage->end_form;
			$webpage->print("</td></tr>\n");
		}
		if ( $SchedLock ) {
			$webpage->print("<tr><td>Load Scheduler State</td><td>\n");
			print $webpage->start_form;
			$webpage->param('command','mon_loadstate');
			$webpage->param('args',"");
			print $webpage->hidden(-name=>'command',);
			print $webpage->hidden(-name=>'args',);
			print $webpage->submit(-name=>"Go");
			print $webpage->end_form;
			$webpage->print("</td></tr>\n");
			$webpage->print("<tr><td>Save Scheduler State</td><td>\n");
			print $webpage->start_form;
			$webpage->param('command','mon_savestate');
			$webpage->param('args',"");
			print $webpage->hidden(-name=>'command',);
			print $webpage->hidden(-name=>'args',);
			print $webpage->submit(-name=>"Go");
			print $webpage->end_form;
			$webpage->print("</td></tr>\n");
		}
	}
	# Reload Auth file
	$webpage->print("<tr><td>Reload Auth</td><td>");
	print $webpage->start_form;
	$webpage->param('command',"mon_reload");
	$webpage->param('args',"auth");
	print $webpage->hidden(-name=>'command',);
	print $webpage->hidden(-name=>'args',);
	print $webpage->submit(-name=>'Go');
	print $webpage->end_form;
	$webpage->print("</td></tr>");
	# Reset Mon
	$webpage->print("<tr><td>Reset Mon</td><td>");
	print $webpage->start_form;
	$webpage->param('command',"moncgi_reset");
	print $webpage->hidden(-name=>'command',);
	print $webpage->submit(-name=>'Go');
	print $webpage->end_form;
	$webpage->print("</td></tr>");
	# Shutdown Mon
	if ( $TermLock == 0 ) {
	$webpage->print("<tr><td>Shutdown Mon</td><td>");
	print $webpage->start_form;
	$webpage->param('command',"moncgi_stop");
	print $webpage->hidden(-name=>'command',);
	print $webpage->submit(-name=>'Go');
	print $webpage->end_form;
	$webpage->print("</td></tr>");
	}
	$webpage->print("</table>\n");
	$webpage->print("</td></tr></table>");
	# NPG End
}

###############################################################
# Mon-specific functions. These all have the common feature that they connect
# to a Mon server and retrieve some data. Generally these functions are called
# by the Presentation functions, or if they are called directly they do no 
# special output formatting.
###############################################################

sub mon_connect {
	# Performs the basic connection, and if necessary, authentication,
	# to a mon server.
	# If successful, returns a 1
	# If unsuccessful because of login failure, returns -1
	# If unsuccessful because of other reasons, returns 0
	my $retval;
	# If we're not connected, we need to connect and possibly authenticate
	if ( (! defined($c->connected()) ) || ( $c->connected() == 0 ) ) {
		$c->connect();
		if ($c->error) {
			$retval = $c->error;
			$connect_failed = 1;	# set the global $connect_failed var
			return 0;
		}
		# Is login required? If so, set the username and password.
		if ($must_login) {
			# Are username and password blank.
			# If so, use the default username/password.
			if (( ( ! defined($loginhash{"username"}) ) && 
			( ! defined($loginhash{"password"}) )) ||
			( ( $loginhash{"username"} eq "" ) && 
			( $loginhash{"password"} eq "" )) ){
				# Login is required but no username/password
				# was given, so try the default account
				$loginhash{"username"} = $default_username ;
				$loginhash{"password"} = $default_password ;
			}
			$c->login(%loginhash);
		}
	}
	if ( ($must_login) && ( defined($c->error) ) && 
	( ($c->error =~ /530 login unsuccessful/) ||
	($c->error =~ /no (password|username)/ ) ) ){
		# Login was required and failed, present the authform if it
		# hasn't already been presented. Since some presentation
		# functions call multiple methods you could easily get
		# multiple prompts, which is confusing to the user.
		&moncgi_authform ($command,"$args") unless $has_prompted_for_auth;
		$has_prompted_for_auth = 1;
		return -1;
	}
	return 1;
}

# NPG
# Connection error presentation sub.
sub mon_connectfail {
	my $retval;
	my $mess;
	# We should be called when a presentation sub detects a connection
	# issue. Preferably through $connect_failed.
	# Try to connect agian so we can get the error.
	my $conn = &mon_connect ;
	if ( $connect_failed == 1 ){
		# Its still down print the error.
		if ($c->error) {
			$retval = $c->error;
			$mess = "<font face=\"$fixed_font_face\">Could not contact mon server &quot;$monhost&quot;:<br/>$retval </font><br/>";
		}
	}
	# It came up, nevermind.
	return ($mess);
}
# NPG End

sub mon_views {
	my ($viewreq);
	my $conn = &mon_connect ;
	# NPG
	if ( ( $conn == 0 ) || ( $conn == -1 ) ) {
		# Connection or Auth Error
		return $conn;
	}else{
		@views = &mon_list("views","");
	# NPG end
		print STDERR "list_views failed" if ($c->error);
		$viewreq = $webpage->param(-name=>"setview");
		if ($viewreq && ($viewreq eq '--all--' || grep(/^$viewreq$/, @views))) {
			$curview = $viewreq;
		} else {
			$curview = $webpage->cookie(-name=>"$vcookie_name");
		}
		# NPG
		if (!$curview) {
			# Its not in the params, its not in the cookie.
			# So lets SET A DEFAULT here.
			# ARRRRGH! How drunk were they when they wrote this 
			# app anyways.
			$curview = "--all--";
		}else{
			if ($curview ne '--all--') {
		# NPG End
				$c->setview($curview);
			}
		# NPG
		}
		# Make sure something always gets into the coockie.
		$vcookie = $webpage->cookie(-name=>"$vcookie_name",
		-value=>"$curview", -expires=>"+1y",
		-path=>"$vcookie_path",-httponly =>'1', );
		# NPG End
		return 1;
	}
}

# NPG
# Unified mon_list sub.
# Since the connection & error reporting is about the same in these subs lets
# consolidate them.
sub mon_list {
	my ($what, $arg) = (@_);
	my ( %rethash, @retarray, $retval );
	my $conn = &mon_connect ;
	if ( ( $conn == 0 ) || ( $conn == -1 ) ) {
		# Connection or Auth Error
		return $conn;
	}else{
		if ( $what eq "group" ) {
			# List all the hosts in a given group.
			# Returns an array of hosts
			@retarray = $c->list_group ($arg);
		}elsif ( $what eq "watch" ) {
			# List all the watches.
			# Returns an array of defined watch groups and services.
			@retarray = $c->list_watch ($arg);
		}elsif ( $what eq "failures" ) {
			# Returns a hash of failures.
			%rethash = $c->list_failures($arg);
		}elsif ( $what eq "successes" ) {
			# Returns a hash of successes
			%rethash = $c->list_successes($arg);
		}elsif ( $what eq "opstatus" ) {
			# Returns a hash of opstatus
			# It should accept an optional anonymous array 
			# argument, of [group, service ...] to only return
			# the opstatus of the group/service pairs you are 
			# interested in.
			# But it doesn't because I am lazy and don't need this 
			# feature right now :)
			#
			# NPG
			# And when I fixed this, stuff hit the fan.
			# If $arg is null Mon::Client returns jack.
			# IMHO Mon::Client should treat the null like
			# list_opstatus();
			%rethash = $c->list_opstatus( [$arg] );
		}elsif ( $what eq "disabled" ) {
			# Lists all the disabled watches, services, and hosts 
			# Returns the result as a hash.
			%rethash = $c->list_disabled();
		}elsif ( $what eq "pids" ) {
			# List all the pid's.
			# Returns an array of stuff.
			@retarray = $c->list_pids;
		}elsif ( $what eq "descriptions" ) {
			# Returns a hash of service descriptions, indexed
			# by watch and service.
			%rethash = $c->list_descriptions;
		}elsif ( $what eq "alert history" ) {
			@retarray = $c->list_alerthist();
		}elsif ( $what eq "scheduler state" ) {
			# Returns an array containing the state of the 
			# scheduler.
			@retarray = $c->list_state();
		}elsif ( $what eq "dtlog" ) {
			# Returns the downtime log as an array of hash 
			# references.
			@retarray = $c->list_dtlog();
		}elsif ( $what eq "views" ) {
			# Returns the views as an array of hash references.
			@retarray = $c->list_views();
		}
		if ($c->error) {
			$retval = $c->error;
#			print "<font face=\"$fixed_font_face\">
#			Could not list $what on mon server 
#			&quot;$monhost&quot;: $retval 
#			(perhaps you don't have permissions in auth.cf?)
#			</font><br/>\n";
			&moncgi_switch_user($retval);
			return undef;
		} else {
			if ( $what eq "group" || $what eq "watch" 
			|| $what eq "pids" || $what eq "alert history"
			|| $what eq "scheduler state" || $what eq "dtlog"
			|| $what eq "views") {
				return @retarray ;
			}elsif ( $what eq "failures" ||  $what eq "successes" 
			|| $what eq "opstatus" || $what eq "disabled"
			|| $what eq "descriptions") {
				return %rethash ;
			}
		}
	}
}
# NPG End

sub mon_reload {
	# Reload mon config file.
	# Right now the only option supported is to reload the auth.cf file.
	my ($what) = (@_);
	print $webpage->hr;
	print $webpage->h2("Reloading mon...");
	my $retval;
	my $conn = &mon_connect ;
	# NPG
	if ( ( $conn == 0 ) || ( $conn == -1 ) ) {
		return 0;
	}else{
	# NPG end
		$retval = $c->reload($what);
		# NPG
		# Unify output generation
		print "<font face=\"$fixed_font_face\">";
		if ($c->error) {
			$retval = $c->error;
			print "Could not reload &quot;$what&quot; mon server on 
			server &quot;$monhost&quot;:<br/> $retval 
			(perhaps you don't have permissions in auth.cf?)";
			&moncgi_switch_user($retval);
		} else {
			print "mon server -- $what on &quot;$monhost&quot; successfully reloaded.";
		}
		print "</font>\n";
		# NPG End
	}
}

sub mon_loadstate {
	# A wrapper function that calls mon_state_mgr with the proper arguments.
	my ($state) = (@_);
	$state = "disabled" if $state eq "";
	print $webpage->hr;
	print $webpage->h2("Loading saved state for $state...");
	&mon_state_mgr("load",$state);
}

sub mon_savestate {
	# A wrapper function that calls mon_state_mgr with the proper arguments.
	my ($state) = (@_);
	$state = "disabled" if $state eq "";
	print $webpage->hr;
	print $webpage->h2("Saving current state for $state...");
	&mon_state_mgr("save",$state);
}

sub mon_state_mgr {
	# Loads or saves state of a mon object specifed by $target. Currently
	# the only object supported by mon for loading/saving state is the
	# state of the scheduler, so using this function with $target="" will
	# load or save the state of the scheduler.
	#
	# The load/save action is specified by the variable $action
	my ($action, $state) = (@_);
	my $retval;
	my $conn = &mon_connect ;
	# NPG
	if ( ( $conn == 0 ) || ( $conn == -1 ) ) {
		return 0;
	}else{
	# NPG end
		if ($action eq "save") {
			($retval = $c->savestate($state)) || ($retval = $c->error);
		}elsif ($action eq "load") {
			($retval = $c->loadstate($state)) || ($retval = $c->error);
		}
		# NPG
		# Unify output generation.
		print "<font face=\"$fixed_font_face\">";
		if ($c->error ne "") {
			print "Could not $action state mon server for state 
			&quot;$state&quot; on server &quot;$monhost
			&quot;:<br/> $retval 
			(perhaps you don't have permissions in auth.cf?)";
			&moncgi_switch_user($retval);
		} else {
			print "$action state succeded on mon server &quot;$monhost&quot;.";
		}
		print "</font>\n";
		# NPG End
	}
}

# Stop or start scheduler
sub mon_schedctl {
	# Stops or starts the scheduler, depending on how it was called.
	# No return value.
	my ($action) = (@_);
	my $retval;
	my $conn = &mon_connect ;
	# NPG
	if ( ( $conn == 0 ) || ( $conn == -1 ) ) {
		return 0;
	}else{
	# NPG end
		if ($action eq "stop") {
			$retval = $c->stop();
		} elsif ($action eq "start") {
			$retval = $c->start();
		}
		if ($c->error) {
			$retval = $c->error;
			print "<font face=\"$fixed_font_face\">
			Could not $action mon server on server 
			&quot;$monhost&quot;:<br/> $retval 
			(perhaps you don't have permissions in auth.cf?)
			</font>\n";
			&moncgi_switch_user($retval);
		}
	}
}

# NPG
# mon_enable & mon_ disable are basically the smae code, so consolidate it
# Will leave the orig. subs as wrappers, like mon_loadstate & mon_savestate
sub mon_entity_mgr {
	my ( $action, $type, $arg1, $arg2) = @_;
	my $retval;
	# Check $action
	if ( $action eq "enable" || $action eq "disable" ) {
		# Validate our input.
		if ( $type eq "service" || $type eq "watch" || $type eq "host" ) {
			if ($type eq "service") {
				if ( !&validate_gs ( $arg1,$arg2 ) ) {
					$webpage->print("<center>Group or Service does not exist!<br/></center>\n");
					return 0;
				}
			} elsif ($type eq "watch") {
				if ( !&validate_group( $arg1 ) ) {
					$webpage->print("<center>Group does not exist!<br/></center>\n");
					return 0;
				}
			}elsif ($type eq "host") {
				# Need to add code to validate host..
			}
			# Only show the rest of this page if we are allowed to see
			# information about this group.
			if ( ($show_watch_strict) && ( $type ne "host") ) {
				unless ( &can_show_group($arg1) ) {
					&moncgi_authfail;
					return 0;
				}
			}
			my $conn = &mon_connect ;
			if ( ( $conn == 0 ) || ( $conn == -1 ) ) {
				return 0;
			}else{
				if ( $action eq "enable" ) {
					print $webpage->h2("Enabling $type...");
					if ($type eq "service") {
						$retval = $c->enable_service($arg1, $arg2);
					} elsif ($type eq "host") {
						$retval = $c->enable_host($arg1);
					} elsif ($type eq "watch") {
						$retval = $c->enable_watch($arg1);
					}
				}else{
					print $webpage->h2("Disabling $type...");
					if ($type eq "service") {
						$retval = $c->disable_service($arg1, $arg2);
					} elsif ($type eq "host") {
						$retval = $c->disable_host($arg1);
					} elsif ($type eq "watch") {
						$retval = $c->disable_watch($arg1);
					}
				}
				print "<font face=\"$fixed_font_face\">";
				if ($c->error) {
					$retval = $c->error;
					print "Could not $action $type with arguments 
					&quot;$arg1&quot; and &quot;$arg2&quot; 
					on server &quot;$monhost&quot;:<br/> $retval 
					(perhaps you don't have permissions in auth.cf?)
					</font>\n";
					&moncgi_switch_user($retval);
					return 0;
				} else {
					print "$action $type succeeded for ";
					if ($type eq "service") {
						print "$arg2";
					}else{
						print "$arg1";
					}
					print "</font>\n";
					return 1;
				}
			}
		}else{
			$webpage->print("<center>Unsupported Object<br/></center>\n");
			return 0;
		}
	}else{
		$webpage->print("<center>Unsupported Action<br/></center>\n");
		return 0;
	}
}
# NPG End

# Enable a disabled object
sub mon_enable {
	my ($arg) = (@_);
	my ($type, $arg1, $arg2) = split (/\,/, $arg);
	&mon_entity_mgr("enable",$type, $arg1, $arg2);
}

# Disable an enabled object
sub mon_disable {
	my ($arg) = (@_);
	my ($type, $arg1, $arg2) = split (/\,/, $arg);
	&mon_entity_mgr ( "disable",$type,$arg1,$arg2);
}

sub mon_test_service {
	# Test a service immediately.
	# Accepts as arguments a group and a service, and optionally an
	# test argument, which can be either monitor, alert, upalert, or 
	# startupalert. Default test is "monitor"
	my ($arg) = (@_);
	my ($group, $service,$test) = split (/\,/, $arg);
	# NPG
	# Validate our input.
	my $Valid = &validate_gs ( $group,$service );
	if ( $Valid == 0 ) {
		&setup_page("Test Service", 400);
		$webpage->print("<center>Group or Service does not exist!<br/></center>\n");
		return 0;
	}elsif ( $Valid == -1 ) {
		&setup_page("Test Service", 403);
		return 0;
	}
	# NPG End
	# Only show the rest of this page if we are allowed to see
	# information about this group.
	if ($show_watch_strict) {
		unless ( &can_show_group($group) ) {
			# NPG
			&setup_page("Test Service", 403);
			# NPG End
			&moncgi_authfail;
			return 0;
		}
	}
	$test = "monitor" if $test eq "";
	# NPG
	# Check for appropriate test values.
	my @tests = qw ( monitor alert startupalert upalert );
	my $testOK = 0;
	foreach my $tmp (@tests) {
		if ( $test eq $tmp ) {
			$testOK++;
		}
	}
	if ( !$testOK ) {
		&setup_page("Test Service", 400);
		return 0;
	}
	&setup_page("Test Service", 200);
	# NPG End
	print $webpage->h2("Performing $test test on service $service in group $group...");
	my $retval;
	my $conn = &mon_connect ;
	# NPG
	if ( ( $conn == 0 ) || ( $conn == -1 ) ) {
		return 0;
	}else{
	# NPG end
		$retval = $c->test($test, $group, $service);
		print "<font face=\"$fixed_font_face\">";
		# NPG
		# Unify output generation.
		if ($c->error) {
			$retval = $c->error;
			print "Could not successfully execute 
			&quot;$test&quot; test service &quot;$service&quot; 
			on hostgroup &quot;$group&quot; 
			on server &quot;$monhost&quot;: $retval 
			(perhaps you don't have permissions in auth.cf?)";
			&moncgi_switch_user($retval);
		} else {
			print "test $test completed for service $service and 
			hostgroup $group:<br/><br/><br/>";
			print "&nbsp;$retval";
		}
		print "</font>\n";
		# NPG end
	}
}

sub mon_test_config {
	# Test the mon config file immediately.
	# Takes no argument (there is only one config file after all)
	print $webpage->h2("Testing the syntax of your mon config file...");
	my $retval;
	my $conn = &mon_connect ;
	# NPG
	if ( ( $conn == 0 ) || ( $conn == -1 ) ) {
		return 0;
	}else{
	# NPG end
		my @s = $c->test_config;
		# NPG
		# Unify output generation.
		$webpage->print("<font face=\"$fixed_font_face\">");
		if ($c->error) {
			$retval = $c->error;
			if ( $retval !~ /^520 test config completed/ ) {	# command not authorized
				print "Could not successfully 
				&quot;test config&quot; on server 
				&quot;$monhost&quot;:<br/>$retval 
				(perhaps you don't have permissions in auth.cf?)";
				&moncgi_switch_user($retval);
			} elsif ( (defined ($s[0])) && ($s[0] == 0) ) {	# there are config file errors
				$webpage->print("Error in config file found:
				<br/><br/>" . $s[1] . "<br/><br/>");
				$webpage->print("You may have other errors in 
				your configuration file. The checking stops 
				after the first one is found.");
			} else {	# some other error occurred
				print "Could not successfully 
				&quot;test config&quot; on server 
				&quot;$monhost&quot;:<br/>$retval 
				(perhaps you don't have permissions in auth.cf?)";
				&moncgi_switch_user($retval);
			}
		} else {
			$webpage->print("Test config completed OK, no errors 
			were found in your config file.");
		}
		$webpage->print("</font>\n");
		# NPG End
	}
}

# Reset mon
sub mon_reset {
	print $webpage->hr;
	print $webpage->h2("Reset mon...");
	my $retval;
	my $conn = &mon_connect ;
	# NPG
	if ( ( $conn == 0 ) || ( $conn == -1 ) ) {
		return 0;
	}else{
		if ( $cgiparams{'keepstate'} eq "on" ) {
	# NPG end
			# specify we want to keep the scheduler state
			# NPG
			# Hardcode keepstate
			$retval = $c->reset("keepstate");
			# NPG End
		} else {
			# reset scheduler state, don't give reset any arguments
			$retval = $c->reset();
		}
		# NPG
		# Unify output generation
		$webpage->print("<font face=\"$fixed_font_face\">");
		if ($c->error) {
			$retval = $c->error;
			$webpage->print("Could not reset mon server on 
			&quot;$monhost&quot;: $retval 
			(perhaps you don't have permissions in auth.cf?)");
			&moncgi_switch_user($retval);
		} else {
			$webpage->print ("Mon server on 
			&quot;$monhost&quot; successfully reset.<br/><br/>
			Monitor states were ");
			if ( $cgiparams{'keepstate'} eq "on" ) {
				$webpage->print ("NOT ");
			}
			$webpage->print ("reset.<br/>");
		}
		# Stop Scheduler Support
		$webpage->print ("Scheduler ");
		if ( $cgiparams{'schedstop'} eq "on" && $SchedCtl == 1 ) {
			$retval = $c->stop();
			if ($c->error) {
				$webpage->print ("NOT ");
			}
		}else{
			$webpage->print ("NOT ");
		}
		$webpage->print ("stopped.");
		$webpage->print ("</font>\n");
		# NPG End
	}
}

# NPG
# Stop the Mon Serve
sub mon_stop {
	print $webpage->hr;
	print $webpage->h2("Halt Mon Server");
	my $retval;
	my $conn = &mon_connect ;
	if ( ( $conn == 0 ) || ( $conn == -1 ) ) {
		return 0;
	}else{
		$retval = $c->term();
	}
	$webpage->print("<font face=\"$fixed_font_face\">");
	if ($c->error) {
		$retval = $c->error;
		$webpage->print("Could not halt mon server on 
			&quot;$monhost&quot;: $retval 
			(perhaps you don't have permissions in auth.cf?)");
			&moncgi_switch_user($retval);
		} else {
			$webpage->print ("Mon server on 
			&quot;$monhost&quot; successfully halted.");
		}
		$webpage->print ("</font>\n");
}
# NPG End

sub mon_list_dtlog {
	# Lists the downtime log (all of it) and returns the results as
	# an array of hash references.
	my ($group, $service) = (@_);
	my $retval ;
	my (@ltmp, @l, $line);
	my (@recovery_times);
	my ($first_failure_time, $mtbf, $mean_recovery_time, $median_recovery_time, $std_dev_recovery_time, $max_recovery_time, $min_recovery_time);
	my $min_recovery_time_default = 9999999999999;	# some arbitrary really big number
	# NPG
	# Call the unified Listing sub.
	my $conn = &mon_connect ;
	if ( !defined ($conn)) {
		return undef;
	}elsif ( ( $conn == 0 ) || ( $conn == -1 ) ) {
		return $conn;
	}else{
		if ( $ListLock ) {
			return undef;
		}else{
			@ltmp = &mon_list("dtlog","") ;
			# NPG end
			my $time_now = time;
			$max_recovery_time = 0;		# initialize this to something really small
			$min_recovery_time = $min_recovery_time_default;	# initialize this to something really big
			$first_failure_time = $time_now;
			# Loop through all downtimes, get first downtime, min and max, 
			# and filter based on criteria/specifications.
			foreach $line (reverse sort {$a->{"failtime"} <=> $b->{"failtime"}}(@ltmp)){
				# Test to see if this is the first failure time.
				if ($line->{"failtime"} < $first_failure_time) {
					# since this list is already sorted, this will only be true
					# the very first time we go thru this loop
					$first_failure_time = $line->{"failtime"};
				}
				# Skip this line if a group and/or service param was
				# specified and the param doesn't match.
				# NPG
				# Consolidate Code.
				if (( defined($group) && ($group ne "") && 
				($group ne $line->{"group"} ) ) || ( 
				defined($service) && ($service ne "") && 
				($service ne $line->{"service"} ) )) {
					next;
				}
				# MAN that'll make your head hurt.
				# NPG End
				# Only show this downtime log entry if we are allowed
				# to see information about this group. This is
				# probably slow but no one said security was efficient!
				if ($show_watch_strict) {
					unless ( &can_show_group($line->{"group"}) ) {
						# This line should not be shown to the user
						next;
					}
				}
				# Add this downtime to the list of downtimes for
				# statistical calculation purposes.
				push(@l, $line);
				push(@recovery_times, $line->{"downtime"});
				# set min and max downtimes
				$min_recovery_time = $line->{"downtime"} if $line->{"downtime"} < $min_recovery_time;
				$max_recovery_time = $line->{"downtime"} if $line->{"downtime"} > $max_recovery_time;
			}
			undef @ltmp; #we don't need @ltmp's memory anymore
			# Calculate mean recovery time
			$mean_recovery_time = &arithmetic_mean(@recovery_times);
			# also calculate median recovery time
			$median_recovery_time = &median(@recovery_times);
			# calculate the mean time between failures as:
			# (total elapsed time since first failure + 
			# E(time until first failure))/(total # of failures)
			$mtbf = (scalar(@recovery_times) == 0) ? 0 : ($time_now - $first_failure_time + (($time_now - $first_failure_time) / scalar(@recovery_times))) / scalar(@recovery_times);
			# Calculate std deviation of failure times
			$std_dev_recovery_time = &std_dev(@recovery_times);
			# In case $max_recovery_time and $min_recovery_time are unset 
			# (i.e. there were no failures), set them to sensible defaults.
			$min_recovery_time = 0 if $min_recovery_time == $min_recovery_time_default;
			return $first_failure_time, scalar(@recovery_times), $mtbf, $mean_recovery_time, $median_recovery_time, $std_dev_recovery_time, $min_recovery_time, $max_recovery_time,  @l ;
		# NPG
		}
		# NPG End
	}
}

sub mon_ack {
	# This subroutine takes a comma separated list of a group and a service
	# as input.
	# It relies on the global "$ackcomment" for its text.
	# If the user is authenticated as anyone other than the default
	# user, mon_ack inserts their name into the ack comment.
	#
	# mon_ack sends an acknowledgement to the mon server for the given 
	# service on the host, the effect of which is to disable any further
	# alerts for the current failure period. 
	# Ack'ing a service that is not in a failure state will produce the
	# following error:
	# "520 service is in a non-failure state"
	#
	# We get around this by only presenting the option to ack for 
	# services that are currently in a failure state.
	my ($args)  = @_;
	my ($group, $service) = split (/\,/, $args);
	# Only show the rest of this page if we are allowed to see
	# information about this group.
	if ($show_watch_strict) {
		unless ( &can_show_group($group) ) {
			&moncgi_authfail;
			return 0;
		}
	}
	my $retval ;
	my $conn = &mon_connect ;
	# NPG
	if ( ( $conn == 0 ) || ( $conn == -1 ) ) {
		return 0;
	}else{
	# NPG end
		my $name_string = ( ($loginhash{"username"} eq $default_username) || ($loginhash{"username"} eq "") ) ? "" : "[$loginhash{'username'}]: ";
		# Ack the service w/comment
		$retval = $c->ack($group, $service, "${name_string}${ackcomment}");
		# NPG
		# Unify output generation.
		$webpage->print("<font face=\"$fixed_font_face\">");
		my $Object = "\"$service\" in group \"$group\"";
		if ($c->error) {
			$retval = $c->error;
			$webpage->print("mon_ack: Could not ack service 
			$Object on mon server &quot;$monhost&quot;: $retval 
			(perhaps you don't have permissions in auth.cf?)");
			&moncgi_switch_user($retval);
		} else {
			$webpage->print("Service $Object acknowledged 
			successfully.");
		}
		$webpage->print("</font>\n");
		# NPG end
	}
}

sub mon_servertime {
	# This subroutine calls the Mon::Client::servertime function and
	# returns the time as a scalar, or undef if there is an error.
	my $retval;
	my $servertime;
	my $conn = &mon_connect ;
	# NPG
	if ( ( $conn == 0 ) || ( $conn == -1 ) ) {
		return 0;
	}else{
	# NPG end
		$servertime = $c->servertime;
		if ($c->error) {
			$retval = $c->error;
			print "<font face=\"$fixed_font_face\">
			Could not get server time on mon server 
			&quot;$monhost&quot;: $retval 
			(perhaps you don't have permissions in auth.cf?)
			</font>\n";
			&moncgi_switch_user($retval);
			return undef;
		} else {
			return $servertime;
		}
	}
}

sub mon_checkauth {
	# This subroutine checks the authorization for a given command to see
	# if it is authorized by the server.
	# Returns:
	#  -1 if a connection with the server cannot be made
	#   0 if the command is not authorized,
	#   1 if the command is authorized
	#   The error string if checkauth fails
	# (which really shouldn't happen unless a Mon server isn't running)
	my ($cmd)  = @_;
	my $retval ;
	my $conn = &mon_connect ;
	# NPG
	if ( ( $conn == 0 ) || ( $conn == -1 ) ) {
		$retval = -1;
	}else{
	# NPG end
		$retval = $c->checkauth($cmd);
		if ($retval == 0) {		# command not authorized
			$retval = 0;
		} elsif ($retval == 1) {	# command authorized
			$retval = 1;
		} else {
			# This command should not fail unless a mon server is not running
			$retval = $c->error;
			$webpage->print("<font face=\"$fixed_font_face\">
			mon_checkauth: Could not check auth for \"$cmd\" 
			on mon server &quot;$monhost&quot;: $retval</font>\n");
		}
	}
	# NPG
	return $retval;
	# NPG end
}

sub mon_state_change_enable_only {
	# This is called only by query_opstatus disabled, and wraps
	# mon_state_change because it assumes only one group and this can
	# span multiple groups (although the command is enable only, which
	# simplifies things some).
	my ($group, $param);
	my @groups = &mon_list ("watch","" ) ;
	foreach $param ( keys(%cgiparams) ) {
		# For each matching action, try to enable or else exit in case
		# of an authentication failure (this makes us keep our CGi
		# variables so that when we do authenticate, all of our
		# actions are executed).
		if ($param =~ /^enagroup_/) {
			&mon_enable("watch,$cgiparams{$param}") || return undef;
		} elsif ($param =~ /^enahost_/) {
			&mon_enable("host,$cgiparams{$param}") || return undef;
		} elsif ($param =~ /^enasvc_/) {
			&mon_enable("service,$cgiparams{$param}") || return undef;
		}
	}
	&query_opstatus("disabled");
}

sub mon_state_change {
	# This function changes one or more of the states of the group given as
	# an argument. It uses the global value %cgiparam to know what else 
	# (hosts and/or services) to modify.
	my ($group, %hosts, $host, %op, $service);
	$group = $cgiparams{'group'};
	my %d = &mon_list("disabled","") ;
	# List the state of the group
	if (!defined $group) {
		$webpage->print("Invalid group \"$group\"\n");
		return undef;
	}
	# Disable/enable group, if that was requested
	# NPG
	# Changed to nested if instead of if && elsif && since the first test 
	# was the same.
	if ( defined ($cgiparams{"group_$group"}) ) {
		if ($cgiparams{"group_$group"} eq "ena" ) {
			# Enable group if the group is already disabled
			if ( ${d{"watches"}{$group}} ) {
				&mon_enable("watch,$group") || return 0;
			}
		} elsif ( $cgiparams{"group_$group"} eq "dis" ) {
			# Disable group if the group is already enabled
			if ( ! ${d{"watches"}{$group}} ) {
				&mon_disable("watch,$group") || return 0;
			}
		}
	}
	# NPG End
	@_ = &mon_list("group", $group) ;
	# List each member of the group, check to see if there is something
	# defined for the group.
	# If so, then change the state if that is required.
	# Don't make a state change if one is not required.
	foreach $host (@_) {
		# Hosts are disabled if they begin with an asterisk
		# (this is mon's convention, not mine)
		if  ($host =~ m/^\*/) {
			# Host is disabled, check to see if we should try and
			# enable
			$host =~ s/^\*//;
			if ( (defined($cgiparams{"host_$host"})) && 
			($cgiparams{"host_$host"} eq "ena") ) {
				# Try to enable and stop if we can't.
				&mon_enable("host,$host") || return 0;
			}
		} else {
			# Host is enabled, check to see if we should try and
			# disable
			if ( (defined($cgiparams{"host_$host"})) && 
			($cgiparams{"host_$host"} eq "dis") ) {
				# Try to disable and stop if we can't.
				&mon_disable("host,$host") || return 0;
			}
		}
	}
	# Check each service on the host to see if its state needs to change.
	%op = &mon_list("opstatus","");
	foreach $service (sort keys %{ $op{$group} }) {
		if ( ( defined($cgiparams{"svc_$service"}) ) &&
		( $cgiparams{"svc_$service"} eq "dis") ) {
			if (! ${d{"services"}{$group}{$service}}) {    
				# service is enabled
				# Try to disable and stop if we can't 
				# (i.e. no permissions)
				&mon_disable("service,$group,$service") || return 0;
			}
		} elsif ( ( defined($cgiparams{"svc_$service"}) ) &&
		( $cgiparams{"svc_$service"} eq "ena") ) {
			if (${d{"services"}{$group}{$service}}) {
				# service is disabled
				# Try to enable and stop if we can't 
				# (i.e. no permissions)
				&mon_enable("service,$group,$service") || return 0;
			}
		}
	}
}

###############################################################
# Mon.cgi-specific functions
# The moncgi_* fuctions generally do not manipulate Mon in any way or present
# any Mon output to the user.
# The moncgi_* functions will each generally call one or more mon_* functions.
###############################################################

# Get the params from the form
sub moncgi_get_params {
	my (@names, $name, $monhost_not_null, $monport_not_null);
	# NPG
	my $tmp;
	# NPG End
	# First get params we know about and expect to get.
	$command = $webpage->param('command');
	$args = $webpage->param('args');
	$rt = $webpage->param('rt');	# return to value for pages which need
					# to keep state info about which page
					# called them.
	$rtargs = $webpage->param('rtargs');		# args for $rt
	$ackcomment = $webpage->param('ackcomment');	# ackcomment
	# For the login form, grab username and password.
	$loginhash{'username'} = $webpage->param('username');
	$loginhash{'password'} = $webpage->param('password');
	# Now get any more parameters which we may have defined.
	@names = $webpage->param;
	foreach $name (@names) {
		$cgiparams{$name} = $webpage->param($name);
	}
	$monhost_not_null = 1 if defined($cgiparams{"h"}) &&  $cgiparams{"h"} ne "";
	$monport_not_null = 1 if defined($cgiparams{"p"}) &&  $cgiparams{"p"} ne "";
	# NPG
	# Allow user to suggest override values of monhost and monport with
	# CGI params.
	# NPG End
	# Untaint monhost and monport, first remove bogus characters,
	# then "officially" untaint them using $1
	if ( defined($monhost_not_null) ) {
		# NPG
		$tmp = $cgiparams{"h"} ;
		$tmp =~ s/[^\w.-]//g;
		$tmp =~ /(.*)/; 
		$tmp = $1;
		if ( scalar @AllowedHosts > 0) {
			if (grep /^$tmp$/, @AllowedHosts ) {
				$monhost = $tmp;
			}
		}else{
			if ($monhost eq $tmp) {
				$monhost = $tmp;
			}
		}
		# NPG End
	}
	if ( defined($monport_not_null) ) {
		# NPG
		$tmp = $cgiparams{"p"} ;
		$tmp =~ s/[^\d]//g;
		$tmp =~ /(.*)/;
		$tmp = $1;
		if ( scalar @AllowedPorts > 0) {
			if (grep /^$tmp$/, @AllowedPorts ) {
				$monport = $tmp;
			}
		}else{
			if ($monhost eq $tmp) {
				$monport = $tmp;
			}
		}
		# NPG End
	}
	# If the user gave either a host or port argument that may (or may 
	# not) override the hard-coded value, we need to preserve this value 
	# and encode it in all future URL's that we generate, so that the 
	# value will be preserved when the mon.cgi page auto-reloads. We also 
	# undef $has_read_config, for mod_perl namespace purposes (scenario: 
	# instance 1 of mon.cgi is invoked with a h= parameter, instance 2 is 
	# not, the instance 2 will eventually pick up instance 1's h= 
	# parameter.
	if ( ( defined($monhost_not_null) ) || ( defined($monport_not_null) ) ) {
		$monhost_and_port_args = "h=$monhost&amp;p=$monport&amp;";
		undef $has_read_config ;
	} else { # user did not enter in either monhost or monport args
		$monhost_and_port_args = "";
	}
}

sub moncgi_logout {
	# This subroutine provides the written evidence to the user that they
	# have logged out of the mon server. The actual logging out is done in
	# the setup_page() routine, when the auth cookie is destroyed in
	# accordance to the value of the global variable $destroy_auth_cookie
	print $webpage->hr;
	print $webpage->h3("User <i>$loginhash{'username'}</i> has been logged off.<br/>");
	print $webpage->h3("You will need to re-authenticate to perform further privileged actions.");
}

# This subroutine presents the authentication form to the user, and runs the
# command the user was trying to execute with the new, level of privilege.
# This subroutine is usually called because a user tried to do something that
# they aren't authorized to do.  The special command name "moncgi_login" is
# given when the user wants to login without actually doing anything that
# requires privs.
sub moncgi_authform {
	my ($command, $args) = (@_);
	# NPG
	print $webpage->start_form(-method=>'POST', );
	# NPG End
	print $webpage->hr;
	# Test to see if the command was the special command "moncgi_login".
	$webpage->print("<center>");
	if ($command eq "moncgi_login") {
		$command = "query_opstatus";
		print $webpage->h3("Please enter your mon username and password below.<br/>");
	} else {
		print $webpage->h3("You must authenticate as a user of sufficient privilege to perform this command.<br/>");
	}
	$webpage->print("</center>");
	# Start the table.
	# NPG
	# Fix this for XHTML Compliance
	$webpage->print("<table class=\"auth\" border=\"0\" align=\"center\">\n");
	# NPG End
	$webpage->print("<tr>\n");
	$webpage->print("<td><b>MON username:</b></td>\n");
	$webpage->print("<td>");
	print $webpage->textfield(-name=>'username', -size=>8,
	-maxlength=>100, );
	$webpage->print("</td></tr>\n");
	$webpage->print("<tr>\n");
	$webpage->print("<td><b>MON password:</b></td>\n");
	$webpage->print("<td>");
	# Reset the passwd param so it's always blank before printing out the
	# password field.
	$webpage->param('password',"");
	print $webpage->password_field(-name=>'password', -size=>8,-value=>"",
	-maxlength=>100, );
	$webpage->print("</td></tr>\n");
	$webpage->print("<tr>");
	# NPG
	# Fix this for XHTML Compliance
	$webpage->print("<td colspan=2 align=\"center\">");
	# NPG End
	print $webpage->submit(-name=>'Login', );
	$webpage->print("</td></tr>\n");
	$webpage->print("</table>\n");
	# if this works correctly, the command will be re-executed with the
	# credentials the user just entered
	$webpage->param('command', $command);
	$webpage->param('args', $args);
	$webpage->param('ackcomment', $ackcomment);
	# Now pass on the rest of the CGI params as hidden fields if there are 
	# any params passed to the form.
	foreach (keys (%cgiparams) ) {
		print $webpage->hidden(-name=>"$_", -value=>"$cgiparams{$_}", );
	}
	print $webpage->end_form;
}

# NPG
# I noticed this error a few times in the code, so consolidated it. Removed
# ref to group, if the user isn't authed to see info. about it, don't tell
# them what it is.
sub moncgi_authfail {
	print $webpage->h3("You are not authorized to see detailed information for this group.");
	print $webpage->h4("Please contact your system administrator for access.");
}
# NPG End

# Generic button function
# Not strictly necessary, but could be useful if you wished to disable certain 
# features of the client or add new ones in a test capacity.
sub moncgi_generic_button {
	my ($title, $command) = (@_);
	print $webpage->hr;
	print $webpage->h2("$title");
	$webpage->print ("(command $command not implemented in this client)\n");
	print $webpage->hr;
}

sub moncgi_switch_user {
	# This subroutine is called after a command fails because a user is
	# authenticated as a user with insufficient privs. to perform the
	# command, so we give the user a chance to re-authenticate as someone
	# of sufficient privilege.
	my ($retval) = (@_) ;
	if ( ($must_login) && ($retval =~ /520 command could not be executed/) ) {
		# User doesn't have permission to perform command
		&moncgi_authform ($command,"$args") unless $has_prompted_for_auth;
		$has_prompted_for_auth = 1;
	}
}

# This subroutine prints out the legend table explaining what colors mean
# what. I broke it out as a separate function so I could experiment with its
# location (top or bottom of table).
# Outputs: always returns 1
sub moncgi_print_service_table_legend {
	$webpage->print("<a name=\"legend\"></a>");
	# NPG
	# Table Width and <span>'s defined in CSS
	$webpage->print("<table class=\"legend\" align=\"center\" border=\"1\" summary=\"Service color legend\">");
	$webpage->print("<tr><td rowspan=\"2\"><font size=\"+1\">
	Service color legend:</font><br/>
	<a href=\"$url#opstatus_tbl\">Back to top</a></td>\n");
	$webpage->print("<td>Unchecked</td><td>Good</td>");
	$webpage->print("<td>Failed - Not Alerting</td>");
	$webpage->print("<td>Failed - Alerting</td>");
	$webpage->print("<td>Disabled</td>");
	$webpage->print("</tr>\n");
	$webpage->print("<tr>");
	$webpage->print("<td bgcolor=\"$unchecked_color\"><span class=\"unchecked\">This</span></td>");
	$webpage->print("<td bgcolor=\"$greenlight_color\"><span class=\"ok\">is</span></td>");
	$webpage->print("<td bgcolor=\"$yellowlight_color\"><span class=\"failing\">my</span></td>");
	$webpage->print("<td bgcolor=\"$redlight_color\"><span class=\"failed\">mon</span></td>");
	$webpage->print("<td bgcolor=\"$disabled_color\"><span class=\"disabled\">cgi  :-)</span></td>");
	$webpage->print("</tr></table>\n");
	# NPG End
	return 1;
}

sub moncgi_list_dtlog_navtable {
	# This function is called by list_dtlog. It takes as arguments 
	# the following: ($url, $group, $service, $sortby, $dtlog_begin, 
	# $dtlog_end, $total_failures, $num_events, %sortby_key).
	# This function doesn't print any downtime log information per 
	# se. It just prints out navigation for the user to browse the 
	# downtime log.
	# This function prints out a table allowing the user to navigate 
	# between the previous N downtime log entries and the next N downtime
	# log entries, where N is a number we derive based on what the range
	# the user asked for, what the maximum entries per page to show is, 
	# and the total number of failures observed.
	# The reason it's a separate function is that we want to have the
	# footer placed at the top and the bottom of the downtime log 
	# table, so we're calling it twice.
	my ($url, $group, $service, $sortby, $dtlog_begin, $dtlog_end, $total_failures, $num_events, %sortby_key) = (@_);
	my ($next_matches_lower, $next_matches_upper, $prev_matches_lower, $prev_matches_upper);
	return ($dtlog_begin, $dtlog_end) if $num_events == 0;   # stop if we returned no downtime events
	# Lower bound on which failures to show as next
	# min($dtlog_end + 1, $total_failures
	$next_matches_lower = ($total_failures < $dtlog_end + 1) ? $total_failures : $dtlog_end + 1;
	# Upper bound on which failures to show as next
	# min($next_matches_lower + $dtlog_max_failures_per_page - 1, $total_failures)
	$next_matches_upper = ( $total_failures < ($next_matches_lower + $dtlog_max_failures_per_page - 1) ) ? $total_failures : ($next_matches_lower + $dtlog_max_failures_per_page - 1) ;
	# now reset $dtlog_end in case we were handed a bogus end point
	$dtlog_end = $next_matches_upper if ($dtlog_end > $num_events);
	# now reset $dtlog_begin in case we were handed a bogus begin point
	$dtlog_begin = $dtlog_end if $dtlog_begin > $num_events;
	# Lower bound on which failures to show as prev
	# max(1, $dtlog_begin  - $dtlog_max_failures_per_page)
	$prev_matches_lower = ($dtlog_begin  -  $dtlog_max_failures_per_page > 0) ? ($dtlog_begin  -  $dtlog_max_failures_per_page ) : 1 ;
	# Start printing the "show previous, show next" table
	# NPG
	# Fix this for XHTML Compliance
	$webpage->print("<table border=\"0\"><tr><td width=\"17%\" align=\"left\">\n");
	# NPG End
	# Only give the option to show previous entries if there are previous
	# entries to show.
	if ( ($prev_matches_lower > 0) && ($dtlog_begin > 1) )  {
		# in the case where ($dtlog_begin - $dtlog_max_failures_per_page)
		# (e.g. we are currently showing entry 9-23 and want to display 1-8)
		if ( ( $dtlog_begin > 1) && ($dtlog_begin - $dtlog_max_failures_per_page < 0) ) {
			$prev_matches_lower = 1;
		}
		# Needs to be min($dtlog_begin -1, $total_failures)
		$prev_matches_upper = ($total_failures < $dtlog_begin - 1) ? $total_failures : $dtlog_begin - 1;
		printf("<a href=\"%s?%scommand=list_dtlog&amp;args=%s,%s,%s,%d,%d\">" , $url, ${monhost_and_port_args}, $group, $service, $sortby, $prev_matches_lower, $prev_matches_upper );
		printf("See events %d-%d</a>\n" , $prev_matches_lower, $prev_matches_upper) ;
	}
	# Print the current entries being shown
	# NPG
	# Fix this for XHTML Compliance
	$webpage->print("</td><td width=\"66%\" align=\"center\"><font size=\"+1\"><b>Displaying downtime events $dtlog_begin-$dtlog_end of $total_failures<br/>(sorting by $sortby_key{$sortby})</b></font></td><td width=\"17%\" align=\"right\">");
	# NPG End
	# Only give the option to show subsequent entries if there are subsequent
	# entries to show.
	if ( ($next_matches_upper <= $total_failures) && ($dtlog_end < $total_failures) ) {
		printf("<a href=\"%s?%scommand=list_dtlog&amp;args=%s,%s,%s,%d,%d\">", $url, ${monhost_and_port_args}, $group, $service, $sortby, $next_matches_lower , $next_matches_upper );
		printf("See events %d-%d</a>\n", $next_matches_lower , $next_matches_upper) ;
	}
	$webpage->print("</td></tr></table>\n");
	# It is possible for these values to be changed in this routine,
	# so send back the values to the calling routine.
	return ($dtlog_begin, $dtlog_end);
}

sub moncgi_test_all {
	# Tests all services for a particular group
	# Inputs: group name
	# Outputs: void
	my ($group) = (@_);
	# Only show the rest of this page if we are allowed to see info about
	# this group.
	if ($show_watch_strict) {
		unless ( &can_show_group($group) ) {
			&moncgi_authfail;
			return 0;
		}
	}
	my (%s, $service);
	# NPG
	# Eliminate the use of $conn as it is only used once.
	if ( !&mon_connect) {
		return 0;
	}
	# NPG end
	%s = &mon_list("opstatus","$group");
	if ( ! &mon_checkauth("test") ) {
		# command is unauthorized
		&moncgi_switch_user("520 command could not be executed");
		return 0;
	}
	foreach $service (keys %{$s{$group}}) {
		&mon_test_service("$group,$service");
	}
	return 1;
}

# This subroutine presents a table for the user to reset with or without
# keepstate. To do nothing is also an option.
sub moncgi_reset {
	# NPG
	if ( $auth_commands{'reset'}{'auth'} == 0 ) {
		$webpage->print("Auth Error");
	}elsif ( $auth_commands{'reset'}{'auth'} == -1 ) {
		$webpage->print("Connect Error");
	}else{
		print $webpage->hr;
		print $webpage->start_form;
		$webpage->param('command','mon_reset');
		$webpage->param('rt', 'moncgi_reset');
		print $webpage->hidden(-name=>'command',);
		print $webpage->hidden(-name=>'rt',);
		$webpage->print("<ul>");
		$webpage->print("<li><input type=\"checkbox\" checked=\"checked\" name=\"keepstate\"/>");
		$webpage->print("Keep monitor state.</li>");
		$webpage->print("<li><input type=\"checkbox\" name=\"schedstop\"/>");
		$webpage->print("Stop Scheduler.</li><li>");
		print $webpage->submit(-name=>"Reset Mon");
		$webpage->print("</li><li><a href=\"$url\">Return to the main page</a>");
		$webpage->print("</li></ul>");
		print $webpage->end_form;
	}
	# NPG End
	return 1;
}

# NPG
sub moncgi_stop{
	if ( $TermLock == 0 ) {
		print $webpage->hr;
		$webpage->print("<p>This command is used to shutdown the Mon process.<br/>");
		$webpage->print("Note: Once Mon has been shutdown, it cannot be restarted via the web interface!");
		print $webpage->start_form;
		$webpage->param('command','mon_stop');
		print $webpage->hidden(-name=>'command',);
		print $webpage->submit(-name=>"Halt Mon");
		print $webpage->end_form;
	}
	return 1;
}
# NPG End

# This subroutine reads and parses the optional mon.cgi config file, and 
# alters global variable values accordingly.
# Inputs: config filename to read
# Outputs: 1      if up-to-date config file has been read
#          0      if config file contains errors.
#          undef  if up-to-date config file was not read (file not found)
sub moncgi_read_cf {
	my ($cf_file) = (@_) ;
	my ($newcf_file_mtime, $key, $val);
	# Test that config file can be read, and if it can, check to see if
	# we've read it.
	if(-r $cf_file) {
		# First we check if we have read a config file. This is
		# controlled by the global variable $has_read_config
		# If so, we check to see if the copy we have read is the
		# latest copy.
		# If not, then we try to read it.
		if ($has_read_config) {
			# We've already read a config file, now check to see
			# if we need to read it again.
			# We re-read the config file if the mtime of the config
			# file is different than the last config file we read.
			$newcf_file_mtime = (stat($cf_file))[9];
			if ($newcf_file_mtime == $cf_file_mtime) {
				return 1;
			} else {
				# Record the new mtime
				$cf_file_mtime = $newcf_file_mtime;
			}
		} else {
			# We've never read the config before, so we get the
			# initial config file mtime.
			$cf_file_mtime = (stat($cf_file))[9];
		}
	} else {
		print STDERR "mon.cgi: moncgi_read_cf: Unable to open config file '$cf_file' for reading: $!\n";
		return undef;
	}
	# Start reading config file
	if ( open (CF , "$cf_file") ) {
		while (<CF>) {
			chomp;	# Skip blank lines and comment lines
			next if /^\s*\#/;
			next if /^\s*$/;
			# Strip off extra blank space at beg. and end of each line
			s/^\s*//;
			s/\s*$//;
			# Parse config file lines
			if (/^(\w+) \s* = \s* (.*) \s*$/ix) {
				$key = $1;
				$val = $2;
				# Trivially untaint $key and $val.
				# We'll do the "real" untainting within the
				# config file parsing, later.
				$key =~ /(.*)/;
				$key = $1;
				$val =~ /(.*)/;
				$val = $1;
				# Look for matching key/value pairs and assign 
				# them to the proper variable. Complain if a 
				# key/value pair doesn't match.
				if ($key eq "organization") {
					$organization = $val;
				} elsif ($key eq "monadmin") {
					$monadmin = $val;
				} elsif ($key eq "logo") {
					$logo = $val ;
				} elsif ($key eq "logo_link") {
					$logo_link = $val ;
				} elsif ($key eq "reload_time") {
					if ($val <= 0) {
						print STDERR "mon.cgi: moncgi_read_cf: $cf_file: $key must be a number > 0\n";
						return 0;
					}
					$reload_time = $val;
				} elsif ($key eq "monhost") {
					# strip out all non-valid chars for taint purposes
					$val =~ s/[^\d\w.-]//g;
					# NPG
					# Allow multiple monhosts.
					if ( !$monhost ) {
						$monhost = $val;
					}else{
						if ($monhost ne $val) {
							@AllowedHosts = ( @AllowedHosts, $val);
						}
					}
					# NPG End
				} elsif ($key eq "monport") {
					# strip out all non-digits for taint purposes
					$val =~ s/[^\d]//g;
					if ($val <= 0) {
						print STDERR "mon.cgi: moncgi_read_cf: $cf_file: $key must be a number > 0\n";
					# NPG
					# Allow multiple monports.
					}else{
						if ( !$monport ) {
							$monport = $val;
						}else{
							if ($monport ne $val) {
								@AllowedPorts = ( @AllowedPorts, $val);
							}
						}
					}
					# NPG End
				} elsif ($key eq "must_login") {
					$must_login = $val;
				} elsif ($key eq "app_secret") {
					$app_secret = $val;
				} elsif ($key eq "default_username") {
					$default_username = $val;
				} elsif ($key eq "default_password") {
					$default_password = $val;
				} elsif ($key eq "login_expire_time") {
					if ($val <= 0) {
						print STDERR "mon.cgi: moncgi_read_cf: $cf_file: $key must be a number > 0\n";
						return 0;
					}
					$login_expire_time = $val;
				} elsif ($key eq "cookie_name") {
					$cookie_name = $val;
				} elsif ($key eq "cookie_path") {
					$cookie_path = $val;
				} elsif ($key eq "fixed_font_face") {
					$fixed_font_face = $val;
				} elsif ($key eq "sans_serif_font_face") {
					$sans_serif_font_face = $val;
				} elsif ($key eq "BGCOLOR") {
					$BGCOLOR = $val;
				} elsif ($key eq "TEXTCOLOR") {
					$TEXTCOLOR = $val;
				} elsif ($key eq "LINKCOLOR") {
					$LINKCOLOR = $val;
				} elsif ($key eq "VLINKCOLOR") {
					$VLINKCOLOR = $val;
				} elsif ($key eq "greenlight_color") {
					$greenlight_color = $val;
				} elsif ($key eq "redlight_color") {
					$redlight_color = $val;
				} elsif ($key eq "yellowlight_color") {
					$yellowlight_color = $val;
				} elsif ($key eq "unchecked_color") {
					$unchecked_color = $val;
				} elsif ($key eq "dtlog_max_failures_per_page") {
					if ($val <= 0) {
						print STDERR "mon.cgi: moncgi_read_cf: $cf_file: $key must be a number > 0\n";
						return 0;
					}
					$dtlog_max_failures_per_page = $val;
				} elsif ($key eq "untaint_ack_msgs") {
					$untaint_ack_msgs = $val;
				} elsif ($key eq "watch") {
					push(@show_watch, $val);
				} elsif ($key eq "show_watch_strict") {
					$show_watch_strict = $val;
				# NPG
				# Add support for our custom vars
				} elsif ($key eq "base_link") {
					$base_link = $val;
				} elsif ($key eq "stylesheet") {
					$stylesheet = $val;
				} elsif ($key eq "logo_pos") {
					$logo_pos = $val;
				# NPG End
				} else {
					print STDERR "mon.cgi: moncgi_read_cf: Unknown key-value pair in config file $cf_file: '$key = $val'\n";
					return 0;
				}
			} else {
				print STDERR "mon.cgi: moncgi_read_cf: Unparseable config file line in config file '$cf_file': $_\n";
				return 0;
			}
		}
	} else {
		print STDERR "mon.cgi: moncgi_read_cf: Unable to open config file '$cf_file' for reading: $!\n";
		return undef;
	}
	# If we've gotten this far, it means the config file was successfully
	# parsed. Set the global variable that indicates that we have read the
	# config file.
	$has_read_config = 1;
	return 1;
}

# This subroutine allows the user to log in without attempting a privileged
# action. Trivial but useful.
sub moncgi_login {
	&moncgi_authform ("moncgi_login","");
}

# This subroutine allows site admins to define their own custom rows of the
# command table, which will appear as a third row to the main command table.
# Take a look at the sample that's here below.
# You're on your own here with whatever code you put in this subroutine!
# Go nuts!
sub moncgi_custom_print_bar {
	# This is a sample routine, contributed by Ed Ravin (eravin@panix.com).
	# Everything is commented out, and none of the functions are implemented 
	# here, but this should give you the idea of what a custom command bar
	# would look like.
	my ($face)= (@_);
#	$webpage->print("<tr>\n");
# NPG
#	$webpage->print("\t<td  align=\"center\"><font face=\"$face\"><a href=$cmd_url=launch&amp;args=space_shuttle>Launch Space Shuttle</a></font></td>\n");
#	$webpage->print("\t<td  colspan=2 align=\"center\"><font face=\"$face\"><a href=$cmd_url=coffee&amp;args=offtime,30,nosugar>Take Coffee Break for 30 minutes</a></font></td>\n");
#	$webpage->print("\t<td  colspan=2 align=\"center\"><font face=\"$face\"><a href=$cmd_url=fizz&amp;args=offtime,0>Reset Soda Machine</font></td>\n");
#	$webpage->print("\t<td  colspan=2 align=\"center\"><font face=\"$face\"><a href=$cmd_url=ackall&amp;args=>Acknowledge All Alarms</a></font></td>\n");
# NPG end
#	$webpage->print("</tr>\n");
}

# This subroutine extends the main command processing loop at the end of this
# script with your own custom commands. Used in conjunction with
# moncgi_custom_print_bar, and other subroutines that you define in this
# script, moncgi_custom_commands provides a nice way to extend mon.cgi at your
# site. From Ed Ravin (eravin@panix.com)
sub moncgi_custom_commands {
	if ($command eq "replace_this_string_with_a_custom_command") {
		&setup_page("Custom Command #1");
		# &call_your_first_custom_code_here;
	}elsif ($command eq "replace_this_string_with_another_custom_command") {
		&setup_page("Custom Command #2");
		# &call_your_second_custom_code_here;
	# as many other custom commands as you care to define go below
	}else{
	# didn't find anything
	return 0;
	}
	1; # did find something, suppress further command processing
}

###############################################################
# Main program
###############################################################
# Instantiate the mon client
$c = new Mon::Client ( host => $monhost, port => $monport, );
# NPG
# Get Basic Capabilities
&mon_connect;
if ($connect_failed) {
	$ListLock = 0;
}else{
	# Set up Cmd Auth Info.
	foreach my $cmd (keys %auth_commands) {
		$auth_commands{$cmd}{'auth'} = &mon_checkauth($cmd);
		$auth_commands{$cmd}{'bgcolor'} = $auth_commands{$cmd}{'auth'} <= 0 ?  "bgcolor=\"$disabled_color\"" : "";
	}
	# Capability Flags
	if ( $auth_commands{'list'}{'auth'} == 1 ) {
		$ListLock = 0;
	}else{
		$ListLock = 1;
	}
	if ( $auth_commands{'start'}{'auth'} == 1 && $auth_commands{'stop'}{'auth'} == 1 ) {
		$SchedCtl = 1;
	}else{
		$SchedCtl = 0;
	}
	if ( $auth_commands{'loadstate'}{'auth'} == 1 && $auth_commands{'savestate'}{'auth'} == 1 ) {
		$SchedLock = 1;
	}else{
		$SchedLock = 0;
	}
	if ( $auth_commands{'term'}{'auth'} == 1 ) {
		$TermLock = 0;
	}else{
		$TermLock = 1;
	}
	# Is the scheduler up or down
	my @scheduler_status = &mon_list("scheduler state","") ;
	if ($scheduler_status[0] == 0 || $connect_failed) {
		$SchedState = 0;
	}else{
		$SchedState = 1;
	}
	# Set up Views.
	if (!$ListLock) {
		&mon_views();
	}
	# Is it a ListCMD
	if (grep /^$command$/, @ListCmds) {
		$ListCmd = 1;
	}
	# Toggle
	if ( $auth_commands{'enable'}{'auth'} == 1 && $auth_commands{'disable'}{'auth'} == 1 ) {
		$Toggle = 1;
	}else{
		$Toggle = 0;
	}
}
# NPG End
if ($command eq "query_opstatus" ){		# Summary opstatus view
	# NPG
	my $OK = 0;
	# opstatus comamnds preflight check
	foreach my $tmp ( sort keys %opstat_cmds) {
		if ( $args eq $opstat_cmds{$tmp}) {
			$OK = 1;
		}
	}
	if (!$OK) {
		&setup_page("Error", 400);
	}else{
		&query_opstatus($args);
	}
	# NPG end
}elsif ($command eq "query_group" ){		# Expand hostgroup.
	&setup_page("Group Expansion");
	&query_group($args);
}elsif ($command eq "list_alerthist"){		# Alert history button.
	&setup_page("List Alert Log");
	&list_alerthist;
}elsif ($command eq "svc_details"){		# View failure details.
	&svc_details($args);
}elsif ($command eq "mon_test_service"){	# Test a service immediately
	&mon_test_service($args);
	sleep 1;	# if we don't sleep here, svc_details will kick off 
			# before the test does and it will look like we aren't 
			# running a test.
	&svc_details($args);
}elsif ($command eq "mon_schedctl"){		# Stop/start the scheduler
	&mon_schedctl ($args);
	# NPG
	# Is the scheduler up or down
	my @scheduler_status = &mon_list("scheduler state","") ;
	if ($scheduler_status[0] == 0 || $connect_failed) {
		$SchedState = 0;
	}else{
		$SchedState = 1;
	}
	if ($rt eq "list_pids") {
		&setup_page("Server Runtime Info.");
		&list_pids($rtargs);
	}else{
		&query_opstatus("summary");
	}
	# NPG End
}elsif ($command eq "list_dtlog"){		# List the downtime log
	&setup_page("List Downtime Log");
	&list_dtlog($args);
}elsif ($command eq "mon_disable"){		# Disable a host/group/service
	&setup_page("Disable alert for object");
	&mon_disable($args);
	if ($rt eq "query_group") {
		&query_group($rtargs);
	# NPG
	# OK, lets FINISH IMPLEMENTING THIS
	}elsif ( $rt eq "svc_details") {
		&svc_details($rtargs);
	# NPG End
	} else {
		&query_opstatus("summary");
	}
}elsif ($command eq "mon_enable"){		# Enable a host/group/service
	&setup_page("Enable alert for object");
	&mon_enable($args);
	if ($rt eq "query_group") {
		&query_group($rtargs);
	# NPG
	# OK, lets FINISH IMPLEMENTING THIS
	}elsif ( $rt eq "svc_details") {
		&svc_details($rtargs);
	# NPG End
	} else {
		# NPG
		&query_opstatus("summary");
		# NPG End
	}
}elsif ($command eq "list_pids"){		# View pid button.
	# NPG
	&setup_page("Server Runtime Info.");
	# NPG End
	&list_pids;
}elsif ($command eq "mon_reset"){		# Reset mon button.
	&setup_page("Reset mon");
	# NPG
	&mon_reset();
	# Is the scheduler up or down
	my @scheduler_status = &mon_list("scheduler state","") ;
	if ($scheduler_status[0] == 0 || $connect_failed) {
		$SchedState = 0;
	}else{
		$SchedState = 1;
	}
	if ($rt eq "moncgi_reset") {
		&moncgi_reset();
	}
}elsif ($command eq "mon_stop"){		# Reset mon button.
	&setup_page("Halt mon");
	&mon_stop();
	# NPG End
}elsif ($command eq "mon_ack"){			# Reset mon button.
	&setup_page("Acknowledge Service Failure");
	&mon_ack($args);			# $ackcomment is a global
	&svc_details($args);
}elsif ($command eq "mon_reload"){		# Reload button.
	&setup_page("Reload Mon");
	&mon_reload($args);
}elsif ($command eq "mon_loadstate"){		# load mon scheduler state
	&setup_page("Load Scheduler State");
	# right now we expect $args to be empty, since loadstate doesn't take
	# any arguments, but someday it might, so we prepare.
	&mon_loadstate($args);
}elsif ($command eq "mon_savestate"){		# save mon scheduler state
	&setup_page("Save Scheduler State");
	# right now we expect $args to be empty, since loadstate doesn't take
	# any arguments, but someday it might, so we prepare.
	&mon_savestate($args);
}elsif ($command eq "moncgi_logout"){		# Log out as auth user.
	$destroy_auth_cookie = "yes";
	&setup_page("Logging out");
	&moncgi_logout;
}elsif ($command eq "mon_state_change") {
	&setup_page("Operation Status: Modify Object");
	&mon_state_change;
	# NPG
	# Do this Right.
	if ($rt eq "query_group") {
		&query_group($rtargs);
	}
	# NPG End
}elsif ($command eq "mon_state_change_enable_only") {
	&setup_page("Operation Status: Enable Object");
	&mon_state_change_enable_only;
}elsif ($command eq "moncgi_test_all") {
	&setup_page("Operation Status: Test All Services In Group");
	&moncgi_test_all($args);
}elsif ($command eq "mon_test_config") {
	&setup_page("Test Mon Config File Syntax");
	&mon_test_config($args);
}elsif ($command eq "moncgi_reset") {
	&setup_page("Reset mon Server");
# NPG
	&moncgi_reset();
}elsif ($command eq "moncgi_stop") {
	$TermCmd = 1;
	&setup_page("HALT mon Server");
	&moncgi_stop();
# NPG End
}elsif ($command eq "moncgi_login") {
	&setup_page("Log In To mon Server");
	&moncgi_login($args);
# NPG
# Unified Log Listing
}elsif ($command eq "list") {
	if ( $args eq "dtlog" ) {
		&setup_page("List Downtime Log");
		&list_dtlog();
	}elsif ( $args eq "alerthist" ) {
		&setup_page("List Alert Log");
		&list_alerthist;
	}else{
		&setup_page("Error", 400);
	}
# NPG End
}elsif ( &moncgi_custom_commands) {
	# Check for user extensions.  The actual custom commands are 
	# processed inside &moncgi_custom_commands.  moncgi_custom_commands 
	# returns non-zero if it finds a command to execute;
}else {					# All else.
	# NPG
	$ListCmd = 1;
	$command = "query_opstatus";
	$args = "summary";
	&query_opstatus($args);
	# NPG End
}
print $webpage->hr;

# Some stuff we keep around for debugging
#print "commands is $command, args is $args<br>\n";   #DEBUG
#print $webpage->dump;   #DEBUG

&end_page;
$c->disconnect();
